#xc8_compiler.py
#SCons Builder Actions for the XC8 PIC compiler

import SCons.Builder
import os.path
import sys
from utilities import build_includes

sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.utilities.paths import convert_path	


def xc8_generator(source, target, env, for_signature):
	"""
	Create an XC8 command line using the parameter defined in 
	the environment
	"""

	arch = env['ARCH']

	incs = []

	args = ['xc8']

	if 'INCLUDE' in env:
		incs = build_includes(env['INCLUDE'])

	src = map(lambda x: convert_path(str(x)), source)
	out = ['-o' + str(target[0])]

	args.extend(make_chip(env))

	if 'XC8FLAGS' in env:
		args.extend(env['XC8FLAGS'])

	args.extend(incs)
	args.extend(make_rom(env))
	args.extend(make_ram(env))
	args.extend(src)
	args.extend(out)

	args = filter(lambda x: x != "", args)

	return " ".join(args)

def xc8_emit_int_files(target, source, env):
	targetdir = str(target[0].get_dir())

	intfiles = map(lambda x: make_int_files(x, targetdir), source)

	intfiles = reduce(lambda x,y: x+y, intfiles, [])

	target += intfiles + make_int_target_files(target[0])

	#Application modules have their own startup.as as a source file
	if "NO_STARTUP" in env:
		return target, source

	target += map(lambda x: os.path.join(targetdir, x), ['startup.lst', 'startup.obj', 'startup.rlf'])
	target += map(lambda x: os.path.join(targetdir, x), ['startup.as'])

	return target, source

def make_chip(env):
	return ['--chip=%s' % env['ARCH'].property('xc8_target')]

def make_rom(env):
	romstart = 0
	romend = -1
	if 'ROMSTART' in env:
		romstart = env['ROMSTART']

	if 'ROMEND' in env:
		romend = env['ROMEND']

	exclude = ","

	if 'ROMEXCLUDE' in env:
		exclude += ",".join(["-%x-%x" % (x[0],x[1]) for x in env['ROMEXCLUDE']])

	if exclude == ",":
		exclude = ""

	ranges = make_pages([romstart, romend])

	#No rom change specified
	#if romstart == 0 and romend == -1:
	#	return [""]

	ranges_txt = ",".join(["%x-%x" % (x[0], x[1]) for x in ranges])

	if romstart >= 0 and romend > 0:
		return ["--ROM=%s" % ranges_txt + exclude]

	raise ValueError("Unknown ROM Range Values: (%x, %x)" % (romstart, romend))	

def same_page(rom, pagesize=2048):
	startp = rom[0] / pagesize
	endp = rom[1] / pagesize

	if startp == endp:
		return True

	return False

def make_pages(rom, pagesize=2048):
	"""
	Take in a contiguous rom range (low, high) and return it divided on page boundaries.
	"""

	if same_page(rom, pagesize):
		return [rom]

	working = [rom[0], rom[1]]
	ranges = []

	while not same_page(working):
		startp = working[0]/pagesize
		enda = (startp + 1)*pagesize - 1

		ranges.append((working[0], enda))
		working[0] = enda+1

	ranges.append(working)

	return ranges


def make_ram(env):
	args = ['default']
	if 'RAMEXCLUDE' in env:
		exc = map(lambda x:"-{:x}-{:x}".format(x[0],x[1]), env['RAMEXCLUDE'])
		args.extend(exc)

	if 'RAMINCLUDE' in env:
		inc = map(lambda x:"+{:x}-{:x}".format(x[0],x[1]), env['RAMINCLUDE'])
		args.extend(inc)

	ramstatement = ",".join(args)

	return ['--RAM=%s' % ramstatement]

def make_int_files(cfile, folder):
	"""
	Given an Scons node representing a c or as file, create the xc8 intermediate targets
	"""
	name,ext = os.path.splitext(os.path.basename(str(cfile)))

	ints = []

	if ext == ".c":
		ints = [name + '.d', name + '.p1', name + '.pre']
	elif ext == ".as":
		ints = [name + '.d', name + '.lst', name + '.pre', name + '.obj', name + '.rlf']

	return map(lambda x: os.path.join(folder, x), ints)

def make_int_target_files(target):
	name,ext = os.path.splitext(os.path.basename(str(target)))
	outdir = str(target.get_dir())

	return map(lambda x: os.path.join(outdir, x), [name+'.as', name+'.cmf',name+'.cof',name+'.hxl',name+'.sdb',name+'.sym', name + '.lst', name + '.obj', name + '.rlf'])


_xc8_hex = SCons.Builder.Builder(
	generator = xc8_generator,
	emitter = xc8_emit_int_files)

def generate(env):
	env['BUILDERS']['xc8'] = _xc8_hex

def exists(env):
	return 1


from collections import namedtuple
from pymomo.utilities import typedargs
import hashlib
from pymomo.exceptions import *

LogDataItem = namedtuple("LogDataItem", ['name', 'type', 'format', 'is_list'])

class LogDefinition:
	def __init__(self):
		self.name = None
		self.data = []
		self.message = None

	def validate(self):
		if not isinstance(self.name, basestring) or len(self.name) == 0:
			return False, "Invalid or missing name"

		if not isinstance(self.message, basestring) or len(self.message) == 0:
			return False, "Invalid or missing log message"

		return True, ""

	def contains_list(self):
		lists = filter(lambda x: x.is_list, self.data)
		return len(lists) > 0

	def assign_types(self, params):
		"""
		Assign types to the list of params if possible

		Given a list of parameters, attempt to match them with
		parameters based on the type information in this LogDefinition.
		If assignment is not possible, raise an InternalError
		"""

		if not self.is_valid_length(params):
			raise InternalError("The number of params does not correspond with the definition", params=params, definitions=self.data)

		if self.contains_list():
			fixed = zip(self.data[:-1], params[:len(self.data)-1])
			list_type = self.data[-1]
			list_params = params[len(fixed):]

			listed = zip([list_type]*len(list_params), list_params)
			return fixed + listed
		
		return zip(self.data, params)

	def is_valid_length(self, params):
		"""
		Check if the parameter list could correspond with this definition
		"""

		if not self.contains_list():
			if len(self.data) != len(params):
				return False

			return True

		#Even if the list is empty there need to be enough params to match
		#the fixed definitions 
		if len(params) < len(self.data) - 1:
			return False

		return True

	def add_data(self, name, type, format, is_list=False):
		if not typedargs.type_system.is_known_type(type):
			raise ArgumentError("Parameter has unknown type", param=name, type=type)

		if format is not None and not typedargs.type_system.is_known_format(type, format):
			raise ArgumentError("Parameter has unknown format", param=name, type=type, format=format)

		definition = LogDataItem(name, type, format, is_list)
		self.data.append(definition)

	def hash(self):
		m = hashlib.md5()
		m.update(self.name)

		h = m.digest()

		l0 = ord(h[-1])
		l1 = ord(h[-2])
		l2 = ord(h[-3])
		l3 = ord(h[-4])

		return (l3 << 24) | (l2 << 16) | (l1 << 8) | l0


import os.path

def build_summary_cmd(target, source, env):
	"""
	Build a text file with the status results for all of the unit tests in sources.
	sources should point to an array of strings corresponding to .status files.
	"""

	some_failed = False

	targets = {}

	for node in source:
		path = str(node)

		name, targ, ext = parse_name(path)
		if ext != '.status':
			print "Ignoring non-status file %s, this file should not be in this list" % path

		if targ not in targets:
			targets[targ] = []

		targets[targ].append((name, path))

	with open(str(target[0]), 'w') as f:
		f.write('Test Summary\n')

		for targ, tests in targets.iteritems():
			num_tests = len(tests)
			results = map(lambda x: test_passed(x[1]), tests)
			tagged_tests = zip(tests, results)

			failed = filter(lambda x: x[1] == False, tagged_tests)
			passed = filter(lambda x: x[1] == True, tagged_tests)

			num_passed = len(passed)

			if num_passed != num_tests:
				some_failed = True

			f.write("\n## Target %s ##\n" % targ)
			f.write("%d/%d tests passed (%d%% pass rate)\n" % (num_passed, num_tests, (num_passed*100/num_tests)))

			for fail in failed:
				f.write("Test %s FAILED\n" % fail[0][0])

	with open(str(target[0]), "r") as f:
		for line in f.readlines():
			print line.rstrip()

	#Raise a build error if some tests failed
	if some_failed:
		return 1

def test_passed(path):
	with open(path, 'r') as f:
		contents = f.read()
		result = contents.lstrip().rstrip()

		if result == 'PASSED':
			return True
		elif result == 'FAILED':
			return False
		else:
			raise ValueError('Invalid value in test status file %s, contents were %s' %(path, result))

def parse_name(path):
	base = os.path.basename(path)
	(name, ext) = os.path.splitext(base)

	(name, target) = name.split('@', 1)

	return (name, target, ext)


import intelhex

def parse_symtab(path):
	lines = [line.strip() for line in open(path)]

	symtab = {}

	for line in lines:
		if line == "%segments":	
			break

		fields = line.split(' ')

		if len(fields) != 5:
			raise ValueError("File did not have a valid format, too many fields in line:\n%s\n" % line)

		name = fields[0]
		addr = int(fields[1], 16)
		sect = fields[3]

		symtab[name] = (addr, sect)

	return symtab

def patch_goto(ih, address, old, new):
	"""
	Given an address in words (2 bytes), make sure that it points to 
	a goto instruction pointing to the old address and if so update
	it to point to the new address.
	"""

	low = ih[address*2]
	high = ih[address*2  + 1]

	instr = (high & 0b111000) >> 3

	if old is not None and instr != 0b101:
		print "Instruction was not a goto at address 0x%x" % address
		return False

	old_addr = (high & 0b111) << 8 | low

	if old is not None and old_addr != old:
		print "Unexpected address found (expected: 0x%x, found 0x%x)" % (old, old_addr)
		return False

	new_high = 0b101000 | ((new >> 8) & 0b111)
	new_low = new & 0xFF

	ih[address*2] = new_low
	ih[address*2 + 1] = new_high

	return True


import SCons.Builder
import sys
import os
sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.hex8 import symbols

def build_symbols(target, source, env):
	symtab = symbols.XC8SymbolTable(str(source[0]))
	symtab.generate_h_file(str(target[0]))
	symtab.generate_stb_file(str(target[1]))

_build_sym_h = SCons.Builder.Builder(
	action = build_symbols
	)

def generate(env):
	env['BUILDERS']['build_symbols'] = _build_sym_h

def exists(env):
	return 1




from pymomo.exceptions import *
from pymomo.utilities.typedargs.annotate import param, returns
from itertools import tee, izip
from collections import namedtuple

def pairwise(iterable):
    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
    a, b = tee(iterable)
    next(b, None)
    return izip(a, b)

def parse_gpsim_log_statement(statement):
	cycle,value = statement.split(' ')

	cycle = int(cycle)
	value = int(value)

	if value != 0 and value != 1:
		raise InternalError("Invalid i2c log statement", statement=statement)

	return cycle, value

StartCondition = namedtuple("StartEvent", ['cycle'])
RepeatedStartCondition = namedtuple("RepeatedStartEvent", ['cycle'])
StopCondition = namedtuple("StopEvent", ['cycle'])
DataByte = namedtuple("DataByte", ['start_cycle', 'end_cycle', 'value', 'acked'])
AddressByte = namedtuple("AddressByte", ['start_cycle', 'end_cycle', 'address', 'is_write', 'acked'])

class I2CAnalyzer:
	"""
	Given a list of the states the clock and data lines at various sample points,
	decode the i2c sequence and convert it to logical operations: start, write, read
	etc.  
	"""

	ClockRisingEdge 	  = 1
	StartCondition		  = 2
	StopCondition 		  = 3
	UnclassifiedCondition = 4

	def __init__(self, states):
		"""
		Take in a list of tuples of the form (cycle, clock, data) where
		cycle is a monotonically increasing cycle counter (time measured
		in arbitrary units), clock is a 0 for low or 1 for high and data 
		is a 0 for low and 1 for high.
		"""

		self.raw_states = [x for x in states]
		self.processed_states = []

		#Process from raw samples to semantic events on the bus
		for last, curr in pairwise(self.raw_states):
			trans = self._classify_state(last, curr)

			if trans != self.UnclassifiedCondition:
				self.processed_states.append((trans, curr[0], curr[2]))

		#Process semantic events into logical events (Start, Byte, Repeated Start, Stop)
		states = self.processed_states
		self.bus_events = []

		while len(states) > 0:
			trans, states = self._process_transaction(states)
			self.bus_events.extend(trans)

	def _is_byte_transmission(self, states):
		if len(states) != 9:
			return False

		for i in xrange(0, 9):
			if states[i][0] != self.ClockRisingEdge:
				return False

		return True

	def _process_byte(self, states):
		bits = map(lambda x: x[2], states)

		byte = 0
		for i in xrange(0, 8):
			byte <<= 1
			byte |= bits[i] & 0x1

		ack = not bool(bits[8])

		return (states[0][1], states[-1][1], byte, ack)

	def _process_address(self, start, end, value, acked):
		addr = value >> 1
		write = not bool(value & 0x1)

		return AddressByte(start, end, addr, write, acked)

	def _process_transaction(self, states):
		"""
		Process a Start, [Byte...], [Repeated Start...], Stop Series
		"""

		trans = []

		if states[0][0] != I2CAnalyzer.StartCondition:
			raise InternalError("Transaction did not start with StartEvent", initial_state = states[0])

		trans.append(StartCondition(states[0][1]))

		i=1
		while i < len(states):
			#Attempt to parse another byte
			if len(states) - i >= 9:
				potential_byte = states[i:i+9]
				if self._is_byte_transmission(potential_byte):
					start, end, value, acked = self._process_byte(potential_byte)
					if isinstance(trans[-1], StartCondition) or isinstance(trans[-1], RepeatedStartCondition):
						trans.append(self._process_address(start, end, value, acked))
					else:
						trans.append(DataByte(start, end, value, acked))

					i += 9
					continue

			#Check if it's a stop or repeated start
			#These may be preceded by rising clock edges to get the 
			#sda value correct
			if states[i][0] == self.StopCondition:
				trans.append(StopCondition(states[i][1]))
				i += 1
			elif states[i][0] == self.ClockRisingEdge and states[i+1][0] == self.StartCondition:
				trans.append(RepeatedStartCondition(states[i+1][1]))
				i += 2
			elif states[i][0] == self.StartCondition:
				trans.append(RepeatedStartCondition(states[i][1]))
				i += 1
			elif states[i][0] == self.ClockRisingEdge and states[i+1][0] == self.StopCondition:
				trans.append(StopCondition(states[i+1][1]))
				i += 2
				break
			elif states[i][0] == self.StopCondition:
				trans.append(StopCondition(states[i][1]))
				i += i
				break
			else:
				raise InternalError("Invalid I2C State", state=states[i])

		if not isinstance(trans[-1], StopCondition):
			raise InternalError("I2C Transaction did not end with a STOP", last_state=trans[-1])

		return trans, states[i:]	

	def _classify_state(self, last, state):
		"""
		Classify the transition that occured between the last state and this one
		"""

		scl_l = last[1]
		scl_c = state[1]
		sda_l = last[2]
		sda_c = state[2]

		#Transitions while scl are high indicate start and stop
		if scl_l == 1 and scl_c == 1:
			if sda_l == 1 and sda_c == 0:
				return I2CAnalyzer.StartCondition
			elif sda_l == 0 and sda_c == 1:
				return I2CAnalyzer.StopCondition
			else:
				return I2CAnalyzer.UnclassifiedCondition

		if scl_l == 0 and scl_c == 1:
			return I2CAnalyzer.ClockRisingEdge

		return I2CAnalyzer.UnclassifiedCondition 

	def _format_start(self, ev):
		return "%d) Start" % ev.cycle

	def _format_repeated_start(self, ev):
		return "%d) Repeated Start" % ev.cycle

	def _format_stop(self, ev):
		return "%d) Stop" % ev.cycle
	
	def _format_data(self, ev):
		a = 'NACK'
		if ev.acked:
			a = 'ACK'

		return "%d) 0x%x %s" % (ev.start_cycle, ev.value, a)

	def _format_address(self, ev):
		a = 'NACK'
		if ev.acked:
			a = 'ACK'

		d = 'READ from'
		if ev.is_write:
			d = 'WRITE to'

		return "%d) %s 0x%x %s" % (ev.start_cycle, d, ev.address, a)

	def _format_event(self, ev):
		if isinstance(ev, StartCondition):
			return self._format_start(ev)
		elif isinstance(ev, StopCondition):
			return self._format_stop(ev)
		elif isinstance(ev, DataByte):
			return self._format_data(ev)
		elif isinstance(ev, AddressByte):
			return self._format_address(ev)
		elif isinstance(ev, RepeatedStartCondition):
			return self._format_repeated_start(ev)
		
		raise InternalError("Unknown event type in i2c log, cannot format", event=ev)

	def format(self):
		"""
		Format this i2c log for printing
		"""

		log = []

		for ev in self.bus_events:
			log.append(self._format_event(ev))

		return "\n".join(log)

	@classmethod
	@param("clock_path", "path", "readable", desc="Path to GPSIM log of i2c clock signal")
	@param("data_path", "path", "readable", desc="Path to GPSIM log of i2c data signal")
	def FromGPSIMLogs(cls, clock_path, data_path):
		with open(clock_path, "r") as f:
			clk = f.readlines()
			clk = [x.rstrip().lstrip() for x in clk if x.rstrip().lstrip() != ""]

		with open(data_path, "r") as f:
			dat = f.readlines()
			dat = [x.rstrip().lstrip() for x in dat if x.rstrip().lstrip() != ""]

		if len(clk) % 2 != 0 or len(dat) % 2 != 0:
			raise InternalError("Invalid gpsim log file lengths were not even", data_length=len(dat), clock_length=len(clk))

		clk = clk[::2]
		dat = dat[::2]

		curr_clock = 1
		curr_data = 1
		curr_cycle = -100

		#Read all of the samples
		clock_samples = [("clock", parse_gpsim_log_statement(x)) for x in clk]
		data_samples = [("data", parse_gpsim_log_statement(x)) for x in dat]
		samples = clock_samples + data_samples

		processed = [(curr_cycle, 1, 1)]

		for s in sorted(samples, key=lambda x: x[1][0]):
			val = s[1][1]
			cycle = s[1][0]

			if cycle < curr_cycle:
				raise InternalError("Non-monotonic cycle counter", sample=s, current_cycle=curr_cycle)

			if s[0] == 'clock' and val != curr_clock:
				#process a clock edge
				curr_cycle = cycle
				curr_clock = val
				sample = (cycle, curr_clock, curr_data)
				processed.append(sample)
			elif s[0] == 'data' and val != curr_data:
				curr_cycle = cycle
				curr_data = val
				sample = (cycle, curr_clock, curr_data)
				processed.append(sample)

		return I2CAnalyzer(processed)


#MIB Proxy Objects

from pymomo.commander.commands import RPCCommand
from pymomo.commander.exceptions import *

class MIBProxyObject (object):
	def __init__(self, stream, address):
		self.stream = stream
		self.addr = address

	def rpc(self, feature, cmd, *args, **kw):
		"""
		Send an RPC call to this module, interpret the return value
		according to the result_type kw argument.  Unless raise keyword
		is passed with value False, raise an RPCException if the command
		is not successful.
		"""


		r = RPCCommand(self.addr, feature, cmd, *args)
		self.stream.send_cmd(r)

		if "result_type" in kw:
			res_type = kw['result_type']
		else:
			res_type = (0, False)

		res = r.parse_result(*res_type)

		if "raise" not in kw or kw['raise'] == True:
			if res['status'] != 0:
				raise RPCException(res['status'], res['error'])

		return res

	def verbose_printer(self, verbose):
		def print_verb(str):
			if verbose:
				print str

		return print_verb


#pricing.py
#An object to define options for which part offers are valid

class OfferRequirements:
	"""
	A simple class to validate whether a price offer object is valid based on
	who offers it, what packaging it comes in and whether it's in stock.
	"""

	def __init__(self, valid_sellers=None, invalid_sellers=[], 
				valid_packages=None, invalid_packages=[], in_stock=True):
		self.valid_sellers = set(self._wrap_single(valid_sellers))
		self.invalid_sellers = set(self._wrap_single(invalid_sellers))
		self.valid_packages = set(self._wrap_single(valid_packages))
		self.invalid_packages = set(self._wrap_single(invalid_packages))
		self.in_stock = in_stock

	def _wrap_single(self, s):
		if (isinstance(s, basestring)):
			return [self._canonicalize_name(s)]

		if s is None:
			return []

		return map(lambda x: self._canonicalize_name(x), s)

	def _canonicalize_name(self, name):
		"""
		Given a name, like a distributor's name, convert it to
		lowercase and remove all spaces,_ or - characters 
		"""

		if name is None:
			return "None"

		return name.lower().replace(' ', '').replace('-', '').replace('_', '')

	def validate(self, offer, quantity):
		if offer.invalid:
			return False

		if offer.moq is not None and quantity < offer.moq:
			return False

		if self._canonicalize_name(offer.seller) in self.invalid_sellers:
			return False

		if self._canonicalize_name(offer.packaging) in self.invalid_packages:
			return False

		if len(self.valid_sellers)>0 and self._canonicalize_name(offer.seller) not in self.valid_sellers:
			return False

		if len(self.valid_packages)>0 and self._canonicalize_name(offer.packaging) not in self.valid_packages:
			return False

		if self.in_stock and quantity > offer.stock_quantity:
			return False

		return True


#Objects for dealing with PCB packages, pads and pins

from pymomo.exceptions import *
from reference import PCBReferenceLibrary
import itertools

class Package:
	"""
	A pcb component package.

	This contains information on the pins and pads in the package
	as well as the package type.
	"""

	def __init__(self, name, pins, pads):
		"""
		Build a package from a name and a list of pins and pads

		- pins should be a list of Pin objects
		- pads should be a list of Pad objects
		- name should be an identifier that will let us look up information
		  about this package in our database of standard packages.
		"""

		ref = PCBReferenceLibrary()

		self.pins = pins
		self.pads = pads

		refined_name, found = ref.find_package(name)

		if found:
			self.name = refined_name
		else:
			self.name = name

	def __str__(self):
		return "%s Package (%d pins, %d pads)" % (self.name, len(self.pins), len(self.pads))

	def __eq__(self, other):
		try:
			if self.name != other.name:
				return False

			for p1, p2 in itertools.izip_longest(self.pins, other.pins, fillvalue=None):
				if p1 != p2:
					return False

			for p1, p2 in itertools.izip_longest(self.pads, other.pads, fillvalue=None):
				if p1 != p2:
					print p1
					print p2
					return False
		except AttributeError:
			return False

		return True

	def __ne__(self, other):
		return not self == other


class Pin:
	"""
	A through hole pin
	"""

	def __init__(self, x, y, drill=None, ring=None):
		if is_quantity(x) and is_quantity(y):
			self.x = x
			self.y = y
		else:
			raise ValidationError("pin position is not specified as a dimensioned quantity", x=x, y=y)

		if drill is not None and not is_quantity(drill):
			raise ValidationError("pin drill size is not specified as a dimensioned quantity", x=x, y=y)

		if ring is not None and not is_quantity(ring):
			raise ValidationError("pin annular ring is not specified as a dimensioned quantity", x=x, y=y)

		self.drill = drill
		self.ring = ring

	def __eq__(self, other):
		try:
			return self.x == other.x and self.y == other.y and self.drill == other.drill and self.ring == other.ring
		except AttributeError:
			return False

	def __ne__(self, other):
		return not self == other


class Pad:
	"""
	A surface mount pad
	"""

	def __init__(self, x, y, width, height):
		if is_quantity(x) and is_quantity(y):
			self.x = x
			self.y = y
		else:
			raise ValidationError("smt pad position is not specified as a dimensioned quantity", x=x, y=y)

		if is_quantity(width) and is_quantity(height):
			self.width = width
			self.height = height
		else:
			raise ValidationError("width and heigh is not specified as a dimensioned quantity", width=width, height=height)

	def __str__(self):
		return "SMT Pad at (%s, %s) size: (%s, %s)" % (str(self.x), str(self.y), str(self.width), str(self.height))

	def __eq__(self, other):
		try:
			return (self.x == other.x) and (self.y == other.y) and (self.width == other.width) and (self.height == other.height)
		except AttributeError:
			return False

	def __ne__(self, other):
		return not self == other

class Placement:
	"""
	A class describing the physical placement and rotation of a PCB component.
	"""

	def __init__(self, x, y, rotation=0.0):
		"""
		Store an x,y location and a rotation (in degrees)
		"""

		if is_quantity(x) and is_quantity(y):
			self.x = x
			self.y = y
		else:
			raise ValidationError("placement position is not specified as a dimensioned quantity", x=x, y=y)

		self.rotation = rotation

	def __eq__(self, other):
		try:
			return self.x == other.x and self.y == other.y and self.rotation == other.rotation
		except AttributeError:
			return False

	def __ne__(self, other):
		return not self == other

	def __str__(self):
		return "(%s, %s) rotated %d degrees" % (str(self.x), str(self.y), self.rotation)


def is_quantity(val):
	if hasattr(val, 'magnitude') and hasattr(val, 'units'):
		return True

	return False



from pymomo.utilities import build
from nose.tools import *
import unittest
import os.path
from pymomo.exceptions import *

def _local_settings():
	folder,last = os.path.split(__file__)
	path = os.path.join(folder, 'test_settings.json')
	return path

def test_loadsettings():
	build.load_settings()

@raises(ArgumentError)
def test_nofile():
	build.load_settings("./Does_not_exist123.hello")

def test_localsettings():
	build.load_settings(_local_settings())

def test_loadfamilies():
	mib12 = build.ChipFamily('mib12', localfile=_local_settings())
	mib24 = build.ChipFamily('mib24', localfile=_local_settings())

@raises(InternalError)
def test_fakefamily():
	build.ChipFamily('mib36')

class TestMIB24Family(unittest.TestCase):
	def setUp(self):
		self.mib24 = build.ChipFamily('mib24', localfile=_local_settings())

	def test_unknown_chip(self):
		with self.assertRaises(ArgumentError):
			self.mib24.find('24f16ka101', 'unknownmodule')

	def test_validate_target(self):
		t1 = "24f16ka101"
		t2 = "24f16ka101/24fj64ga306"
		t3 = "24fj64ga306/test"
		t4 = "24fj64ga306/unknown"
		t5 = "hello"

		eq_(self.mib24.validate_target(t1), True)
		eq_(self.mib24.validate_target(t2), True)
		eq_(self.mib24.validate_target(t3), True)
		eq_(self.mib24.validate_target(t4), False)
		eq_(self.mib24.validate_target(t5), False)

	def test_properties(self):
		target2 = self.mib24.find('24fj64ga306')
		target = self.mib24.find('24fj64ga306/test')

		target.property('defines')
		target.property('unknown', default=None)

		with self.assertRaises(ArgumentError):
			target.property('unknown')

		defs1 = target.property('defines')
		defs2 = target2.property('defines')

		assert defs1['__PIC24FJ64GA306__'] == 1
		assert defs2['__PIC24FJ64GA306__'] == 1
		assert defs1['__TEST__'] == 1

		with self.assertRaises(KeyError):
			a = defs2['__TEST__']

	def test_module_target(self):
		target = self.mib24.find('24fj64ga306/test', 'mainboard')
		target.property('linker')

		with self.assertRaises(ArgumentError):
			self.mib24.find('24fj64ga306/test', 'unknown_module')


from pymomo.utilities import build
from nose.tools import *
import unittest
import os.path
import os
import platform
from pymomo.exceptions import *
from pymomo.utilities.paths import MomoPaths

class TestSettingsDirectory:
	def setUp(self):
		self.paths = MomoPaths()
		self.confdir = os.path.join(os.path.expanduser('~'), '.config')

	def test_settings_dir(self):
		assert os.path.exists(self.paths.settings)

	@unittest.skipIf(platform.system() != 'Linux', 'Linux specific test')
	def test_settings_linix(self):
		settings_dir = os.path.abspath(os.path.join(self.confdir, 'WellDone-MoMo'))

		assert settings_dir == self.paths.settings

	@unittest.skipIf(platform.system() != 'Windows', 'Windows specific test')
	def test_settings_windows(self):
		if 'APPDATA' in os.environ:
			base = os.environ['APPDATA']
		else:
			base = self.confdir

		settings_dir = os.path.abspath(os.path.join(base, 'WellDone-MoMo'))
		assert settings_dir == self.paths.settings

	@unittest.skipIf(platform.system() != 'Darwin', 'Mac OS X specific test')
	def test_settings_darwin(self):

		settings_dir = os.path.abspath(os.path.join(os.path.expanduser('~'), 'Library', 'Preferences',
															'WellDone-MoMo'))

		assert settings_dir == self.paths.settings


from unit_test import UnitTest, find_sources
import pic24
import os
import sys
import unit_test

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))
from pymomo.exceptions import *
from pymomo.utilities import paths

class Pic24ModuleTest (UnitTest):
	"""
	A unit test for the PIC24 series of microprocessors that tests a single .c/.h module
	providing mocks for any other modules that the module under test (MUT) references.
	"""

	def __init__(self, files, **kwargs):
		self.includes = []
		self.additional_sources = []

		UnitTest.__init__(self, files, **kwargs)

		self.processed_dirs = set()

		modname = self._build_module_name(files[0])
		self._find_module(modname)
		for name in self.extra_modules:
			self._find_module(name)
		
	def _build_module_name(self, testfile):
		testfile = os.path.basename(self.files[0])
		if not testfile.startswith('test_'):
			raise InternalError('Module unit test does not have an appropriate filename: %s' % testfile)

		modname = os.path.splitext(testfile)[0][5:]
		return modname
	
	def _find_module(self, modname):
		if self._find_in_dir(modname) == False:
			found = False
			for src in self.additional_sources:
				found = self._find_in_dir(modname, src)
				if found:
					break

			if not found:
				raise InternalError("Could not find module %s for module unit test %s" % (modname, self.name))

	def _find_in_dir(self, modname, srcdir=None):
		if srcdir is None:
			testdir = os.path.dirname(self.files[0])
			srcdir = os.path.abspath(os.path.join(testdir, '..', 'src'))

		incdirs, sources, headers = find_sources(srcdir)
		normsrc = os.path.normpath(srcdir)

		if normsrc not in self.processed_dirs:
			self.includes += incdirs
			self.processed_dirs.add(normsrc)

		if modname not in sources:
			return False

		self.files.append(sources[modname])
		return True

	def _parse_target(self, target):
		return target

	def _parse_sources(self, value):
		"""
		Parse an additional source directory other than simply src
		"""

		basedir = os.path.dirname(self.files[0])
		srcpath = os.path.normpath(os.path.join(basedir, value))
		self.additional_sources.append(srcpath)

	def _parse_modules(self, value):
		"""
		For module unit tests where the test just compiles one or several modules, allow
		the user to specify extra modules that need to be compiled in.
		"""

		parsed = value.split(',')
		files = map(lambda x: x.rstrip().lstrip(), parsed)
		self.extra_modules = files

	def build_target(self, target, summary_env):
		statusnode = pic24.build_moduletest(self, target)
		summary_env['TESTS'].append(statusnode)
			
unit_test.known_types['module'] = Pic24ModuleTest


#!/usr/bin/env python
#pcbtool
#A program for creating BOMs, gerbers and assembly drawings from annotated EAGLE circuit board diagrams.

import sys
import os.path
import os
import shutil

import cmdln
from colorama import init, Fore, Style
from pymomo.utilities.paths import MomoPaths
from pymomo.pybom import board,pricing
from pymomo import pyeagle
from pymomo.utilities.template import RecursiveTemplate
import math 
import itertools

init()

class PCBTool(cmdln.Cmdln):
	name = 'pcbtool'
	def __init__(self):
		self.paths = MomoPaths()
		self.get_identifiers()

		cmdln.Cmdln.__init__(self)

	@cmdln.option('-o', '--output', action='store', help='Output directory to hold created files.')
	@cmdln.option('-p', '--paste', action='store_true', default=False, help='Store the solder paste layer (default: no).')
	def do_fab(self, subcmd, opts, id):
		"""${cmd_name}: Generate fabrication and assembly files for this board.

		Create all of the files required for assembly and fabrication based on this board file.  Board file must be a 2-layer board with metadata tagging as described in the MoMo documentation.
		
		${cmd_usage}
        ${cmd_option_list}
		"""

		if os.path.exists(opts.output) and not os.path.isdir(opts.output):
			self.error("Invalid output location, seems to be a file: %s" % opts.output)
		elif not os.path.exists(opts.output):
			path = os.path.abspath(opts.output)
			os.makedirs(path)

		brd = self.find_identifier(id)
		pyeagle.build_production(brd, opts.output, opts.paste)

	def do_list(self, subcmd, opts):
		"""${cmd_name}: List all known PCB board ids.
		
		${cmd_usage}
        ${cmd_option_list}
		"""
		brds = self.paths.select(self.paths.pcb, filter=lambda x, y, z: z == '' or z == '.brd', include_dirs=True)

		print "Listing all known board ids under <momo_root>/pcb directory"
		print "You may specify any of these ids to select the .brd file shown"
		print "\nBoards:"
		for id, path in sorted(self.ids.iteritems(), key=lambda x:x[0]):
			print Fore.BLUE + "%s:" % id + Style.RESET_ALL + " '%s'" % os.path.relpath(path, start=self.paths.pcb)

		print ""

	@cmdln.option('-v', '--variant', action='store', default='MAIN', help="Assembly variant to process.")
	@cmdln.option('-o', '--output', action='store', help='Output file path, defaults to stdout')
	@cmdln.option('--prices', action='store_true', default=False, help='Include prices in BOM')
	@cmdln.option('-n', '--units', action='store', default=1, type=int, help="Number of units to quote")
	@cmdln.option('-d', '--dist', action='append', default=None, help="Only use the given distributors (e.g. Digi-Key)")
	@cmdln.option('--no-dist', action='append', dest='nodist', default=[], help="Exclude the listed distributors.")
	@cmdln.option('-p', '--packaging', action='append', default=[], help="Only allow the specified packaging (e.g. cut tape).")
	@cmdln.option('--no-package', dest='nopackage', action='append', help="Disallow the specified packaging type (e.g. Custom Reel)")
	@cmdln.option('--no-stock', dest='nostock', action='store_false', default=True, help="Allow out of stock offers.")
	@cmdln.option('-f', '--format', action='store', choices=['csv', 'pdf'], default='csv')
	def do_bom(self, subcmd, opts, id):
		"""${cmd_name}: Create a BOM for the specified assembly variant.

		${cmd_usage}
        ${cmd_option_list}
		"""

		reqs = self.build_pricemodel(opts)
		brd = self.get_board(id)

		if opts.format == 'pdf' or opts.format == 'html':
			data = brd.variant_data(opts.variant, include_costs=opts.prices, cost_quantity=opts.units, requirements=reqs)
			print data
			#templ = RecursiveTemplate('bom_template.html')
			#templ.add(data)
			#formatted = templ.format_temp()

			#shutil.move(formatted, opts.output)

	@cmdln.option('-v', '--variant', action='append', default=None, help="Print detailed information about the listed assembly variants.")
	def do_info(self, subcmd, opts, id):
		"""${cmd_name}: Print information about the board file indicated.

		If one or more variant is indicated, print detailed information about those
		assembly variants including the number of line items, etc. 
		all may be passed to --variant to indicate that all assembly variants should
		be printed in detail.

		${cmd_usage}
        ${cmd_option_list}
		"""

		brd = self.get_board(id)

		print "Board Name: " + brd.partname
		print "Dimensions: %sx%s inches" % (brd.height, brd.width)
		print "Number of Assembly Variants: %d" % len(brd.variants)
		print "Assembly Variant Names:", brd.variants.keys()

		if len(brd.unknown) == 0:
			print Fore.GREEN + "All parts matched with part numbers" + Style.RESET_ALL
		else:
			print Fore.RED + "Unmatched parts exist:" + Style.RESET_ALL, brd.unknown

		if opts.variant is None:
			return

		if opts.variant[0] == 'all':
			opts.variant = brd.variants.keys()

		print "\n---Assembly Variant Details---\n"

		for var in opts.variant:
			if var not in brd.variants:
				print Fore.RED + "Variant %s does not exist" + Style.RESET_ALL

			lines = len (brd.variants[var])
			items = sum([len(x) for x in brd.variants[var]])

			print "Variant Name: %s" % var
			print "Distinct BOM Lines: %d" % lines
			print "Total Part Count: %d" % items
			print "Through-Hole Parts: %d" % sum([len(x) for x in brd.variants[var] if x[0].package_info()['pins']>0])
			print "Total Through-Hole Pins: %d" % sum(map(lambda x: x[0].package_info()['pins']*len(x), brd.variants[var])) 
			print "SMD Parts: %d" % sum([len(x) for x in brd.variants[var] if x[0].package_info()['pads']>0])
			print "Total SMD Pads: %d" % sum(map(lambda x: x[0].package_info()['pads']*len(x), brd.variants[var]))
			print ""

	@cmdln.option('-e', '--excess', action='store', type=float, default=0.0, help="Excess percentage to purchase of each component, in %. (rounded up)")
	@cmdln.option('-o', '--output', action='store', help='Output directory to hold created files.')
	def do_order(self, subcmd, opts, *boards):
		"""${cmd_name}: Create an order list that can be pasted into a distributor website for ordering.  Board variants added to the
		order should be specified by giving 3 items:
		<board id> <variant> <number>

		All boards will have the same excess applied.

		${cmd_usage}
        ${cmd_option_list}
		"""

		excess = opts.excess/100.0 + 1

		if len(boards) % 3 != 0:
			self.error("You must specify 3 items per board <id> <variant> <number of units>")

		parts = []

		for i in xrange(0, len(boards), 3):
			id = boards[i]
			var = boards[i+1]
			num = int(boards[i+2])
			brd = self.get_board(id)

			quant = int(math.ceil(num*excess))

			print "Processing %s (%s): %d units with %.0f%% excess" % (id, var, num, opts.excess)
			parts.extend(brd.export_list(var)*quant)

		out,close = self.build_outstream(opts)

		distinct = [(k, len(list(g))) for k,g in itertools.groupby(sorted(parts))]

		for k,g in distinct:
			print "%d, %s" % (g,k) 

		if close:
			out.close()

	@cmdln.option('-v', '--variant', action='store', default='MAIN', help="Assembly variant to quote.")
	@cmdln.option('-n', '--units', action='store', default=1, type=int, help="Number of units to quote")
	@cmdln.option('-e', '--excess', action='store', type=float, default=0.0, help="Excess percentage to purchase of each component, in %. (rounded up)")
	@cmdln.option('-l', '--lines', action='store_true', default=False, help="Show cost of each BOM line")
	@cmdln.option('-d', '--dist', action='append', default=None, help="Only use the given distributors (e.g. Digi-Key)")
	@cmdln.option('--no-dist', action='append', dest='nodist', default=[], help="Exclude the listed distributors.")
	@cmdln.option('-p', '--packaging', action='append', default=[], help="Only allow the specified packaging (e.g. cut tape).")
	@cmdln.option('--no-package', dest='nopackage', action='append', help="Disallow the specified packaging type (e.g. Custom Reel)")
	@cmdln.option('--no-stock', dest='nostock', action='store_false', default=True, help="Allow out of stock offers.")
	def do_quote(self, subcmd, opts, id):
		"""${cmd_name}: Quote a BOM price for this board at the given volume.

		The price is looked up using the Octopart API.  For this to work all parts must be tagged in the EAGLE brd file as described in the MoMo documentation and you must have a valid octopart API key specified in either a config file or the environment

		${cmd_usage}
        ${cmd_option_list}
		"""
		brd = self.get_board(id)

		if opts.variant not in brd.variants:
			self.error("Invalid variant passed (%s), options are: %s" % (opts.variant, str(brd.variants.keys())))

		excess = opts.excess/100.0 + 1
		units = opts.units
		multiplier = units*excess

		model = self.build_pricemodel(opts)

		prices, unmatched = brd.price_variant(opts.variant, multiplier, model)

		for line in unmatched:
			print Fore.RED + "Could not find:", map(lambda x: x.name, line), Style.RESET_ALL

		print "Price for %d Units with %.0f%% excess" % (units, opts.excess)
		total_price = 0.0

		for i, line in enumerate(prices):
			parts, offer = line
			price = float(offer['price'])*multiplier
			if opts.lines:
				desc = "from %s" % offer['offer'].seller
				if offer['offer'].packaging is not None and offer['offer'].packaging is not "":
					desc += " in %s" % (offer['offer'].packaging,)
				
				print "Line %d: $%.2f (%d @ $%.2f) %s" % (i+1, price, len(parts), float(offer['price']), desc), map(lambda x: x.name, parts)
			
			total_price += price

		print "Total price: $%.2f" % total_price
		print "Unit price: $%.2f" % (total_price/units)

	def build_pricemodel(self, opts):
		"""
		Given the arguments passed in opts build a PricingRequirements object that meets those requirements
		"""

		valid_sellers = []
		invalid_sellers = []
		valid_packages = []
		invalid_packages = []
		in_stock = True

		if hasattr(opts, 'dist'):
			valid_sellers = opts.dist

		if hasattr(opts, 'nodist'):
			invalid_sellers = opts.nodist

		if hasattr(opts, 'packaging'):
			valid_packages = opts.packaging

		if hasattr(opts, 'nopackage'):
			invalid_packages = opts.nopackage

		if hasattr(opts, 'nostock'):
			in_stock = opts.nostock

		return pricing.OfferRequirements(	valid_sellers=valid_sellers, invalid_sellers=invalid_sellers,
											valid_packages=valid_packages, invalid_packages=invalid_packages,
											in_stock=in_stock)


	def build_outstream(self, opts):
		"""
		If -o was passed, open that file for writing, otherwise return stdout.  Return a bool indicating if the 
		caller should close the stream when finished.
		"""

		if opts.output is None:
			return (sys.stdout, False)

		try:
			stream = open(opts.output, "w")
		except IOError:
			print "Could not open output file %s for writing." % opts.output
			sys.exit(1)

		return (stream, True)

	def get_identifiers(self):
		ids = self.paths.select(self.paths.pcb, filter=lambda x, y, z: z == '.brd')
		
		#Get all of the parent directory names. If they only correspond to a single .brd file,
		#they can also be used as identifiers
		parent_dirs = map(lambda x: (os.path.basename(os.path.dirname(x)), x), ids)
		seen_dirs = set()
		ambig_dirs = set()

		for d in parent_dirs:			
			if d in seen_dirs:
				ambig_dirs.add(d[0])
			else:
				seen_dirs.add(d)

		valid_dirs = seen_dirs.difference(ambig_dirs)
		brd_ids = map(lambda x: (os.path.basename(x), x), ids)

		ids = valid_dirs | set(brd_ids)

		self.ids = {x[0]: x[1] for x in ids}
		self.ambiguous_ids = ambig_dirs

	def error(self, text):
		print Fore.RED + "Error Occurred" + Style.RESET_ALL
		print text
		sys.exit(1)

	def param_error(self, type, param, fix):
		"""
		Print an error message and then quit.  Print type: param.\nfix
		"""

		print Fore.RED + "%s: " % type + Style.RESET_ALL + "%s\n" % param + fix
		sys.exit(1)

	def find_identifier(self, id):
		"""
		Given a string that may correspond to either a .brd file name or a folder, find if there is an
		unambiguous way to return the correct .brd file corresponding to that id.
		"""

		#Check if id is a path that points to valid file
		if os.path.isfile(id):
			return id
		elif os.path.isfile(os.path.join(self.paths.pcb, id)):
			return os.path.join(self.paths.pcb, id)

		if id in self.ambiguous_ids:
			self.param_error("Ambiguous Board Name Passed", id, "Try specifying the .brd file name directly or use pcbtool list to see valid identifiers.")

		if id not in self.ids:
			self.param_error("Unknown Board Name Passed", id, "Specify either the .brd file name or its parent directory if not ambiguous. You can also specify either an (absolute or relative from cwd) path to a .brd file or a relative path from <momo_root>/pcb")

		return self.ids[id]

	def get_board(self, id):
		"""
		Given a board identifier, get the pybom Board object corresponding to it or quit with an error.
		"""

		brdfile = self.find_identifier(id)
		
		try:
			brd = board.Board.FromEagle(brdfile)
		except ValueError as e:
			self.error(str(e))

		return brd

def main():
	pcbtool = PCBTool()
	return pcbtool.main()


import loghex
import logcaller
import logassert
import logfinished
import logcheckpoint

#Map control bytes to statement handlers
statements = {
		0: logfinished.LogFinished,
		1: loghex.LogHex,
		#2: LogString,
		3: logcaller.LogCaller,
		4: logassert.LogAssert,
		5: logcheckpoint.LogCheckpoint
}


#A threaded implementaton of asyncronous line oriented
#IO.  This allows a python to program to read from a
#pipe that is not seekable or peekable in a cross-platform
#way without potentially blocking forever.

from threading import Thread, Event
from cStringIO import StringIO
from Queue import Queue
import os
from pymomo.exceptions import *

class AsyncLineBuffer:
	def __init__(self, file, separator='\n', strip=True):
		"""
		Given an underlying file like object, syncronously read from it 
		in a separate thread and communicate the data back to the buffer
		one byte at a time.
		"""

		self.queue = Queue()
		self.items = Event()
		self.thread = Thread(target=AsyncLineBuffer.ReaderThread, args=(self.queue, file, self.items, separator, strip))
		self.thread.daemon = True
		self.thread.start()

	@staticmethod
	def ReaderThread(queue, file, items, separator, strip):
		while True:
			line_done = False
			line = ''
			while not line_done:
				c = file.read(1)
				line += c

				if line.endswith(separator):
					line_done = True

			if strip:
				line = line[:-len(separator)]

			queue.put(line)
			items.set()

	def available(self):
		"""
		Return the number of available lines in the buffer
		"""
		
		return self.queue.qsize()

	def readline(self, timeout=3.0):
		"""
		read n bytes, timeout if n bytes are not available 
		"""

		if self.available() == 0:
			flag_set = self.items.wait(timeout)
			if flag_set is False:
				raise TimeoutError("Asynchronous Read timed out waiting for a line to be read")

			self.items.clear()

		return self.queue.get()



#bool.py
#Simple boolean type

def convert(arg, **kwargs):
	if arg is None:
		return arg

	if isinstance(arg, basestring):
		comp = arg.lower()
		if comp == 'true':
			return True
		elif comp == 'false':
			return False
		else:
			raise ValueError("Unknown boolean value (should be true or false): %s" % arg)

	return bool(arg)

def default_formatter(arg, **kwargs):
	return str(arg)



#formatting.py

def indent_block(string_block, level):
	indent = ' '*level
	repstr = '\n' + indent

	retval = string_block.replace('\n', repstr)
	return indent + retval

def indent_list(list, level):
	"""
	Join a list of strings, one per line with 'level' spaces before each one
	"""

	indent = ' '*level
	joinstr = '\n' + indent

	retval = joinstr.join(list)
	return indent + retval


#exceptions.py

class TimeoutException:
	def __init__(self, where):
		self.message = "Timeout occured in %s" % where
		self.where = where

class RPCException:
	def __init__(self, type, data):
		self.type = type
		self.data = data

	def __str__(self):
		return "RPC failed, code %s : %s" % (self.type, self.data)

class NoSerialConnectionException:
	def __init__(self, ports ):
		self.message = "No port specified and no valid USB device detected."
		self.ports = ports

	def __str__(self):
		return self.message

	def available_ports(self):
		return self.ports


#externalsim.py
#A SimWorker that encapsulates an external program where commands are
#fed in using stdin and results are collected using stdout

from simworker import SimWorker
import subprocess
from pymomo.utilities.typedargs.annotate import *
from pymomo.exceptions import *
from pymomo.utilities.asyncio import AsyncLineBuffer

class ExternalSimulator (SimWorker):
	"""
	A SimWorker that encapsulates an external simulator program, sending
	commands to stdin and reading results from stdout.  This class provides
	many convenience routines for interacting with the program in reliable ways.
	"""

	def _sim_path(self):
		"""
		Subclasses override to return an array with the
		path and arguments to be used to create a subprocess
		for this simulator
		"""

		raise APIError("ExternalSimulator subclasses must override _sim_path() or _build_sim()")

	def _initialize(self):
		"""
		Subclasses can override in order to perform specific simulator
		initialization routines here.  This is called from _build_sim
		"""

		pass

	def _process_message(self, msg, cmd=None):
		"""
		Subclasses can override in order to process the messages that are returned from the
		simulator after executing a command.
		"""

		return msg

	def _build_sim(self):
		"""
		Create a subprocess for the external simulator and wire up its input and output for us to control
		"""
		

		simargs = self._sim_path()

		if not hasattr(self, 'prompt'):
			raise APIError("Subclasses of ExternalSimulator must define an attribute called prompt")

		self.sim = None
		self.sim = subprocess.Popen(simargs, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
		self.input = AsyncLineBuffer(self.sim.stdout, separator=self.prompt, strip=True)
		self.waiting = False #Whether or not we are waiting for a prompt to appear
		self.waiting_cmd = None

		self._wait_prompt(noparse=True) #discard messages because its just random information
		self._initialize()

	def finish(self):
		if self.sim is not None:
			self.sim.terminate()

	def ready(self):
		if not self.waiting:
			return {'result': True}

		try:
			self._wait_prompt(0.0, cmd=self.waiting_cmd)
		except TimeoutError:
			return {'result': False}

		return {'result': True}

	def wait(self, timeout):
		if not self.waiting:
			return None

		res = self._wait_prompt(timeout)

		return {'result': res}

	def _wait_prompt(self, timeout=3, noparse=False, cmd=None):
		"""
		Wait for a prompt to appear indicating that the last command finished. 
		"""

		#if this returns we know that the end of buffer is self.prompt
		#so we can splice it off
		
		msg = self.input.readline(timeout)
		self.waiting = False

		if not noparse:
			parsed = self._process_message(msg, cmd=cmd)
			return parsed

	def _command(self, cmd, *args, **kwargs):
		"""
		Send the command cmd to the simulator and wait for a prompt
		to appear again signaling that it has finished executing. if
		additional args are passed, they are converted to strings,
		separated with spaces and appended to cmd, which must be a 
		string.
		"""

		if len(args) > 0:
			argstr = " ".join(args)
			cmd = cmd + " " + argstr

		if cmd[-1] != '\n':
			cmd += '\n'

		self.sim.stdin.write(cmd)

		if "sync" not in kwargs or kwargs["sync"]:
			noparse = kwargs.get('noparse', False)
			result = self._wait_prompt(noparse=noparse, cmd=cmd)
			return result
		else:
			self.waiting_cmd = cmd
			return None



#partcache.py
#A simple expiring cache built on top of sqlite

import sys
import os.path

from pymomo.utilities.paths import MomoPaths
from time import time
import sqlite3
import cPickle

overriden_cache = None
overriden_expiry = None

class PartCache(object):
	"""
	A simple persistent key-value store with expiration times

	Persistence comes from a sqlite backend and each time the 
	file is opened, all entries are checked to see if they should
	be expired and deleted if so.
	"""

	DefaultCachePath = os.path.join(MomoPaths().settings, 'pcb_part_cache.db')
	DefaultExpiry = 1*24*60*60		#Default expiration date is 1 day

	def __init__(self, cache=None, no_expire=None, expiration=None):
		if cache is None:
			if overriden_cache is not None:
				cache = overriden_cache
			else:
				cache = PartCache.DefaultCachePath

		if expiration is None:
			expiration = PartCache.DefaultExpiry
		else:
			expiration = int(expiration)

		self.connection = sqlite3.connect(cache)
		self.cursor = self.connection.cursor()

		self.file = cache
		self.expiration = expiration

		self._setup_table()

		if no_expire is None:
			if overriden_expiry is None or overriden_expiry is False:
				self._expire_old()
		elif not no_expire:
			self._expire_old()

	def _setup_table(self):
		query = 'create table if not exists PartCache (key TEXT PRIMARY KEY, created INTEGER, part blob);'
		self.cursor.execute(query)
		self.connection.commit()

	def _expire_old(self):
		query = "delete from PartCache where strftime('%%s','now') - created > %d" % self.expiration
		self.cursor.execute(query)
		self.connection.commit()

		return self.cursor.rowcount

	def size(self):
		"""
		Return the number of entries in this cache
		"""

		query = 'select count(*) from PartCache'
		self.cursor.execute(query)
		return self.cursor.fetchone()[0]

	def _check_valid(self, obj):
		exp = obj['expiration']

		cur = time()

		if exp > cur:
			return True

		return False

	def get(self, id):
		query = 'select part from PartCache where key is ?'
		self.cursor.execute(query, (id,))

		val = self.cursor.fetchone()
		if val is None:
			raise KeyError("id not in cache: %s", str(id))
		
		return cPickle.loads(str(val[0]))

	def try_get(self, id):
		try:
			return self.get(id)
		except KeyError:
			return None

	def set(self, id, obj):
		now = time()
		data = cPickle.dumps(obj)

		query = "insert into PartCache values (?, strftime('%s','now'), ?)"
		self.cursor.execute(query, (id, sqlite3.Binary(data)))
		self.connection.commit()


def default_cachefile(path):
	"""
	Set the default cache file used to back PartCache objects
	"""

	global overriden_cache
	overriden_cache = path

def default_noexpire(do_expire):
	"""
	Set whether cache should not expire by default
	"""

	global overriden_expiry
	overriden_expiry = do_expire



#utilities.py

from SCons.Script import *
from SCons.Environment import Environment
import os
import fnmatch
import json as json
import sys
import os.path
import pic12
import StringIO

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from pymomo.utilities import build
from pymomo.mib.config12 import MIB12Processor

def find_files(dirname, pattern):
	"""
	Recursively find all files matching pattern under path dirname
	"""

	matches = []
	for root, dirnames, filenames in os.walk(dirname, followlinks=True):
		print dirnames, filenames
		for filename in fnmatch.filter(filenames, pattern):
			matches.append(os.path.join(root,filename))

	return matches

def build_includes(includes):
	if isinstance(includes, basestring):
		includes = [includes]

	return ['-I"%s"' % x for x in includes]

def build_libdirs(libdirs):
	if isinstance(libdirs, basestring):
		libdirs = [libdirs]

	return ['-L"%s"' % x for x in libdirs]

def build_staticlibs(libs, chip):
	if isinstance(libs, basestring):
		libs = [libs]

	processed = []
	for lib in libs:

		#Allow specifying absolute libraries that don't get architectures
		#appended
		if lib[0] == '#':
			processed.append(lib[1:])
		else:
			#Append chip type and suffix
			proclib = "%s_%s" % (lib, chip.arch_name())
			processed.append(proclib)

	return ['-l%s' % x for x in processed]

def build_defines(defines):
	return ['-D%s=%s' % (x,str(y)) for x,y in defines.iteritems()]

def get_family(fam, modulefile=None):
	return build.ChipFamily(fam, modulefile=modulefile)

class BufferedSpawn:
	def __init__(self, env, logfile):
		self.env = env
		self.logfile = logfile

		self.stderr = StringIO.StringIO()
		self.stdout = StringIO.StringIO()

	def spawn(self, sh, escape, cmd, args, env):
		cmd_string = " ".join(args)

		print cmd_string
		self.stdout.write(cmd_string)
		
		try:
			retval = self.env['PSPAWN'](sh, escape, cmd, args, env, sys.stdout, sys.stderr)
		except OSError, x:
			if x.errno != 10:
				raise x

			print 'OSError Ignored on command: %s' % cmd_string

		return retval



from pymomo.commander import transport, cmdstream
from pymomo.commander.proxy import *
from pymomo.commander.exceptions import *
from pymomo.utilities.console import ProgressBar
from pymomo.hex16.convert import *
from time import sleep
from tempfile import *
import os
import sys


def reflash_module(controller, hexfile, name=None, address=None, force=False, verbose=True, noreset=False):
	"""
	Given a controller instance, reflash a pic12 application module
	given either its address or name.
	"""

	#Make sure the module exists before pushing the firmware
	mod = controller.get_module(name, address, force)

	controller.clear_firmware_cache()
	bucket = controller.push_firmware(hexfile, 0, verbose=verbose)

	mod.rpc(1, 0, 8, bucket)
	mod.reset()

	sleep(1.5)
	if not controller.alarm_asserted():
		print "Module reflash NOT DETECTED.  Verify the module checksum to ensure it is programmed correctly."
		raise RuntimeError("Could not reflash module, reflash not detected using alarm pin.")

	print "Reflash in progress"
	while controller.alarm_asserted():
		sys.stdout.write('.')
		sys.stdout.flush()
		sleep(0.1)

	print "\nReflash complete."

	sleep(0.5)

	if not noreset: 
		if verbose:
			print "Resetting the bus..."

		controller.reset(sync=True)

def _convert_hex24(hexfile):
	tmpf = NamedTemporaryFile(delete=False)
	tmpf.close()

	tmp = tmpf.name

	out = unpad_pic24_hex(hexfile)
	out.write_hex_file(tmp)
	return tmp

def reflash_controller(controller, hexfile, verbose=True):
	"""
	Given a path to a hexfile, push it onto the controller and then
	tell the controller to reflash itself.
	"""

	processed = _convert_hex24(hexfile)
	controller.push_firmware(processed, 5, verbose=verbose)
	os.remove(processed)
	controller.reflash()

	sleep(0.5)
	if not controller.alarm_asserted():
		print "Controller reflash NOT DETECTED.  You may need to try the recovery procedure."
		raise RuntimeError("Could not reflash controller")

	print "Reflash in progress"
	while controller.alarm_asserted():
		sys.stdout.write('.')
		sys.stdout.flush()
		sleep(0.1)

	print "\nReflash complete."



import collections
import string

def convert(arg, **kwargs):
	if arg is None:
		return None

	if isinstance(arg, BOMLine):
		return arg
	elif isinstance(arg, collections.Sequence) and not isinstance(arg, basestring):
		return BOMLine(arg)

	raise ValueError("Creating a BOMLine from any type other than a BOMLine object or list of parts is not supported")

def default_formatter(arg, **kwargs):
	return str(arg)

class BOMLine:
	digits = frozenset(string.digits)

	def __init__(self, parts):
		self.parts = parts

		self.count = len(parts)
		proto = parts[0]

		self.manu = proto.manu
		self.mpn = proto.mpn
		self.package = proto.package
		self.desc = proto.desc
		self.dist = proto.dist
		self.distpn = proto.distpn
		self.value = proto.value
		self.refs = sorted([x.name for x in parts], key=self._ref_number)
		self.type = self._extract_ref_type()

		if len(self.refs) > 0:
			self.lowest = self._ref_number(self.refs[0])
		else:
			self.lowest = 0

	def _ref_number(self, id):
		"""
		Extract the numeric part of a reference number
		"""

		return int(''.join([c for c in id if c in BOMLine.digits]))

	def _extract_ref_type(self):
		if len(self.refs) == 0:
			return None

		r = self.refs[0]

		for i in xrange(0, len(r)):
			if not r[i].isalpha():
				break

		return r[:i]

	def __str__(self):
		return "%d %s: %s" % (self.count, self.desc, self.refs)



#reference.py
#A manager for reference information used in creating circuit board BOMs
#and production files

import json
import os.path
import re
import sys

from pymomo.utilities.config import ConfigFile

data_file = ConfigFile('pcb_library')

class PCBReferenceLibrary:
	def __init__(self):
		self.lib = data_file

		self._import_packages()
		self._import_descriptions()
		self.valued_types = set(self.lib["types_with_values"])
		self._alpha_re = re.compile(r"^[a-zA-Z]*")

	def _import_packages(self):
		pkgs = self.lib['footprint_names']

		self.packages = {}

		for name,val in pkgs.iteritems():
			self.packages[name] = set(val)

	def _import_descriptions(self):
		self.descs = self.lib['description_templates']

	def find_description(self, name, value):
		"""
		Look to see if there is a template for generating the description
		of this piece based on its reference type R, C, etc and its value
		"""
		t = self._alpha_re.match(name)

		if t is None:
			return None

		cat = t.group(0)

		if cat in self.descs:
			return self.descs[cat].format(value=value)
		
		return None

	def has_value(self, name):
		"""
		Look up if this type has a meaningful value and return True if so, 
		otherwise False.
		"""
		
		t = self._alpha_re.match(name)

		if t is None:
			return False

		cat = t.group(0)

		if cat in self.valued_types:
			return True

		return False

	def find_package(self, pkg):
		"""
		Look for the footprint in our list of known packages.  Return a tuple
		of (package name, bool found).
		"""
		for name, val in self.packages.iteritems():
			if pkg in val:
				return (name, True)

		return (pkg, False)


#fixtures.py

import pytest
from pymomo.commander.exceptions import *

@pytest.fixture
def controller():
	"""
	Return a controller object attached to the first FTDI adapter plugged into 
	this computer.
	"""

	from pymomo.commander.meta.initialization import get_controller

	try:
		con = get_controller()
	except InitializationException as e:
		pytest.skip("Cannot find FSU to communicate with hardware")

	return con


import proxy12
from pymomo.commander.exceptions import *
from pymomo.commander.types import *
from pymomo.commander.cmdstream import *
from pymomo.utilities.console import ProgressBar
import struct
from intelhex import IntelHex
from time import sleep

class MultiSensorModule (proxy12.MIB12ProxyObject):
	ranges = {'offset': set(range(0, 256)), 'gain1': set(range(0, 128)), 'gain2':set(range(0,8)), 'delay': set(range(1, 256)), 
				'invert': set(['yes', 'no']), 'select':set(['current', 'differential'])}
	mapper = {'yes': 1, 'no': 0, 'current': 1, 'differential': 0}
	ids = {'offset': 0, 'gain1': 1, 'gain2': 2, 'select': 3, 'invert': 4, 'delay': 5}
	integral = set(['offset', 'gain1', 'gain2', 'delay'])

	def __init__(self, stream, addr):
		super(MultiSensorModule, self).__init__(stream, addr)
		self.name = 'MultiSensor Module'

	def set_parameter(self, name, value):
		if name not in self.ranges:
			raise KeyError("Invalid parameter options are: %s" % str(self.ranges.keys()))

		if name in self.integral:
			value = int(value)

		valid = self.ranges[name]
		if value not in valid:
			raise ValueError("Invalid parameter value for key %s" % name)

		if not isinstance(value, int):
			if value not in self.mapper:
				raise ValueError("Invalid parameter value for key %s, cannot be mapped to a known integer" % name)

			value = self.mapper[value]

		self.rpc(20, 3, self.ids[name], value)

	def set_sensor_push(self, push):
		"""
		Set whether the sensor module should push sensor data to the controller.
		"""

		res = self.rpc(20, 11, int(push))

	def read_voltage(self):
		res = self.rpc(20, 5, result_type=(1,False))
		return res['ints'][0]

	def acquire_pulses(self):
		"""
		Tell the multisensor module to sample flow rate for 1 sampling period.
		"""

		self.rpc(20, 6)

	def read_pulses(self):
		"""
		Return a list of all of the pulse widths and intervals between pulses
		acquired during the last acquisition period.

		Returns a 2-tuple containing the list of valid pulses and the
		number of invalid pulses
		"""

		res = self.rpc(20, 7, result_type=(2, False))
		counts = res['ints'][0]
		invalid = res['ints'][1]

		pulses = []
		for i in xrange(0, counts):
			res = self.rpc(20, 8, i, result_type=(2, False))
			width = res['ints'][0]
			interval = res['ints'][1]

			if i == counts-1:
				interval = 0

			pulses.append((width, interval))

		return pulses, invalid

	def read_median(self):
		"""
		Find the median of the list of pulse intervals recorded
		and return it.  This call is not idempotent since finding
		the median required destroying the list of recorded intervals
		"""
		res = self.rpc(20, 10, result_type=(1,False))

		return res['ints'][0]

	def read_periods(self):
		"""
		Return the number of periods that have been sampled.
		"""

		res = self.rpc(20, 9, result_type=(1, False))

		return res['ints'][0]




def parse_args(fmt, args):
	"""
	Parse the argument list according to the passed format string.
	each character in fmt specifies the type of each argument.  Valid 
	choices are:
	i - dec, oct, bin or hex integer with leading decoration to distinguish (8 bit)
	a - integer address (16 bit) 
	"""
	if len(args) != len(fmt):
		raise ValueError("Invalid number of arguments, expected %d from format '%s', received %d." % (len(fmt), fmt, len(args)))

	parsed = []

	for i,c in enumerate(fmt):
		argstr = args[i]

		arg = None

		if c == 'i':
			arg = int(argstr, 0)
			if arg >= 256:
				raise ValueError("8-bit integer argument too large: %d" % arg)
		elif c == 'a':
			arg = int(argstr, 0)
			if arg >> 16 != 0:
				raise ValueError("16-bit address argument too large: %d" % arg)

		if arg is None:
			raise ValueError("Unknown argument type: %s", c)

		parsed.append(arg)

	return parsed


from exceptions import TimeoutException

class CMDStream:
	"""
	Send command frames over a transport layer and receive the response
	This converts a byte oriented stream to a frame oriented one.
	"""

	#Types of frame markers
	ACK = 0x06
	NACK = 0x15
	term_chars = [ACK, NACK]

	OkayResult = 0
	ErrorResult = 1

	def __init__(self, transport):
		self.trans = transport

	def parse_term(self, char):
		term = ord(char)

		if term == CMDStream.ACK:
			result = CMDStream.OkayResult
		elif term == CMDStream.NACK:
			result = CMDStream.ErrorResult
		else:
			raise ValueError("Invalid terminator character encountered: %d" % term)

		return result

	def read_frame(self):
		buffer, tchar = self.trans.read_until(CMDStream.term_chars)
		return buffer, self.parse_term(tchar)

	def send_cmd(self, cmd):
		"""
		Given a cmd string, append a newline if required and send it, waiting for
		the response.
		"""

		#Convert the command to a unicode object
		if not isinstance(cmd, basestring) and hasattr(cmd, "__str__"):
			cmdstr = cmd.__str__()
		else:
			cmdstr = cmd

		if cmdstr[-1] != '\n':
			cmdstr += '\n'

		self.trans.write(cmdstr)

		#If there is a custom result handler, use that, otherwise read a standard frame
		if hasattr(cmd, 'handle_result'):
			return cmd.handle_result(self)
		else:
			return self.read_frame()

	def heartbeat(self):
		"""
		Send a heartbeat character on the line that the FSU should respond to with
		the same character if it is working correctly.  Return true if the heartbeat
		was received, return false otherwise.
		"""


		self.trans.write(chr(255))
		
		c = self.trans.read()
		if len(c) == 0 or c[0] != chr(255):
			print "'%s'" % c
			return False 

		return True

	def reset(self):
		self.trans.write(chr(0))
		c = self.trans.read()
		if len(c) == 0 or c[0] != chr(1):
			print "'%s'" % c
			return False 

		return True


from externalsim import ExternalSimulator

import subprocess
from pymomo.utilities.typedargs.annotate import *
from pymomo.exceptions import *
from pymomo.utilities.asyncio import AsyncLineBuffer
from collections import namedtuple

#Known Error codes
messages = {
	'Unable to find processor ': '_process_processor_error',
	'the program file type does not contain processor': '_process_no_proc_error',
	'#No such file or directory': '_process_not_found_error',
	'Warning: Logging can\'t be enabled until a cpu has been selected': '_process_no_proc_error'
}

ignored = [
	'you need to specify processor with the processor command',
	'SetProcessorByType FIXME',
	'Extended linear address',
	'Disabling WDT',
	'Leaving pic_processor::LoadProgramFile',
	'running...'

]

InvalidProcessorMessage = namedtuple('InvalidProcessorMessage', ['processor'])
NoProcessorMessage = namedtuple('NoProcessorMessage', [])
NotFoundMessage = namedtuple('NotFoundMessage', ['path'])

def match_message(line, msg):
	if msg[0] == '#':
		msg = msg[1:]
		return msg in line

	return line.startswith(msg)

class GPSIM (ExternalSimulator):
	"""
	A Simworker object encapsulating the gpsim pic12 and pic16 simulator from gputils.
	"""

	prompt = '**gpsim> '
	
	def _sim_path(self):
		return ['gpsim', '-i']

	def _init_sim(self):
		pass

	def _process_message(self, msg, cmd=""):
		if not msg.startswith(cmd):
			raise InternalError("Command was not echoed in simulator response.")

		msg = msg[len(cmd):].rstrip()

		lines = [x for x in msg.split('\n') if len(x) > 0]

		parsed = []

		for line in lines:
			matches = [messages[x] for x in messages.iterkeys() if match_message(line, x)]
			if len(matches) == 0:
				ignore_matched = [x for x in ignored if match_message(line, x)]

				if len(ignore_matched) > 0:
					continue

				raise APIError("Unknown message from gpsim", line=line)
			elif len(matches) > 1:
				raise APIError("Multiple matches for message from gpsim", line=line)

			parser = getattr(self, matches[0])
			parsed.append(parser(line))
		
		return parsed

	def _set_processor(self, proc):
		"""
		Set the processor type for this simulator instance
		"""

		result = self._command('processor', str(proc))
		if len(result) != 0:
			raise InternalError("Error setting processor", msgs=result)


	def set_param(self, name, value):
		"""
		Set the named paramater to the specified value 
		"""

		if name == 'processor':
			self.processor = value
			self._set_processor(value)
		else:
			raise InternalError("Invalid parameter", name=name, value=value)

	def load_program(self, program, type):
		if type == 'elf':
			raise InternalError("ELF files are not supported by gpsim")

		result = self._command('load', program)
		if len(result) != 0:
			raise InternalError("Error loading program", msgs=result)
	
	def execute(self):
		self._command('run', sync=False)
		self.waiting = True

	def raw_command(self, command):
		result = self._command(command)

	def set_log(self, file):
		result = self._command("log on '%s'" % file)
		if len(result) != 0:
			if any(isinstance(x, NoProcessorMessage) for x in result):
				raise InternalError("You must set a processor and load a hex file first")

			raise InternalError("Error adding log", msgs=result)

	#Error code processing logic
	def _process_processor_error(self, line):
		skip = len('Unable to find processor ')
		procname = line[skip:]

		return InvalidProcessorMessage(procname)

	def _process_no_proc_error(self, line):
		return NoProcessorMessage()

	def _process_not_found_error(self, line):
		path,msg = line.rsplit(':')
		if msg != ' No such file or directory':
			raise APIError("Unexpected message processing file not found error", msg=msg, line=line)

		return NotFoundMessage(path)


import unittest
import os.path
import os
from nose.tools import *
from pymomo.exceptions import *
from pymomo.pcb import CircuitBoard
import pymomo.pcb.partcache
import tempfile
import shutil
import atexit
import hashlib

class TestBOMCreation(unittest.TestCase):
	"""
	Test to make sure that BOM matching and BOM generation work.
	"""

	def _create_tempfile(self, src=None):
		"""
		Create a named temporary file that is a copy of the src file
		and register for it to be deleted at program exit.
		"""
		dst = tempfile.NamedTemporaryFile(delete=False)

		if src is not None:
			with open(src, "r+b") as f:
				shutil.copyfileobj(f, dst)

		dst.close()

		atexit.register(os.remove, dst.name)
		return dst.name

	def _hash_file(self, path):
		hasher = hashlib.sha256()

		with open(path,'r') as f:
			block = 2**16 - 1
			buf = f.read(block)
			while len(buf) > 0:
				hasher.update(buf)
				buf = f.read(block)

		return hasher.hexdigest()

	def setUp(self):
		self.srcbrd = self._create_tempfile(os.path.join(os.path.dirname(__file__), 'eagle', 'controller_missing_attrs.brd'))
		cachefile = self._create_tempfile(os.path.join(os.path.dirname(__file__), 'eagle', 'pcb_part_cache.db'))
		
		#Make sure we use our cache file and don't let it expire so we don't delete all of the entries
		pymomo.pcb.partcache.default_cachefile(cachefile)
		pymomo.pcb.partcache.default_noexpire(True)
		self.board = CircuitBoard(self.srcbrd)
		self.board.set_match_engine("CacheOnlyMatcher")

	def tearDown(self):
		pass

	def test_cached_matching(self):
		"""
		Make sure that the cache is working 
		"""

		self.board.match_status()
		print self.board.match_details()

		assert self.board.match_status() == True

	def test_lookup(self):
		part = self.board.lookup('U5')
		assert part is not None

	def test_update_metadata(self):
		self.board.update_metadata('U5')

		board1 = CircuitBoard(self.srcbrd)
		part = board1.find('U5')
		
		print part.mpn
		print part.manu
		print part.desc

		assert part.mpn == 'M25PX80-VMN6TP'
		assert part.manu == 'MICRON'

	def test_update_all_metadata(self):
		self.board.update_all_metadata()

		board1 = CircuitBoard(self.srcbrd)
		
		for part in board1._iterate_parts():
			assert part.mpn is not None
			assert part.manu is not None

	def test_export_excel(self):
		"""
		Make sure that the BOM exporter works for excel files without error
		"""

		output = self._create_tempfile()

		self.board.export_bom(output, format='excel')
		#We can't verify the output with a hash since there's an embedded timestamp
		#in the file format so just make sure it runs without error.




from utils import *

class RetlwInstruction:
	def __init__(self, args):
		self.args = parse_args("i", args)

	def __str__(self):
		return "retlw 0x%x" % self.args[0]

	def encode(self):
		return (0b110100 << 8) | self.args[0]


from pymomo.commander.exceptions import *
import struct

class ModuleDescriptor:
	"""
	A structure holding information about a MIB module
	"""

	def __init__(self, buffer, addr):
		if len(buffer) != 11:
			raise TypeError('ModuleDescriptor', 'Length should have been 11, was %d' % len(buffer))

		hw_type, mod_type, info, name, feat_cnt = struct.unpack('BBB7sB', buffer)

		self.hw = hw_type
		self.type = mod_type
		self.mib_revision = info & 0x0F
		self.flags = info >> 4
		self.name = name
		self.num_features = feat_cnt
		self.address = addr


from pymomo.utilities.typedargs import type_system
from pymomo.utilities import typedargs
from nose.tools import *
from pymomo.exceptions import *

def test_simplereturntype():
	@typedargs.return_type("string")
	def returns_string():
		return "hello"

	val = returns_string()
	formed = type_system.format_return_value(returns_string, val)

	assert formed == "hello"

def test_complexreturntype():
	@typedargs.return_type("map(string, integer)")
	def returns_map():
		return {"hello": 5}

	val = returns_map()
	formed = type_system.format_return_value(returns_map, val)

	assert formed == "hello: 5"



#octopart.py
#Wrapper arround Octopart REST API

import os
import json
import urllib
from pymomo.pcb.bom_matcher import BOMMatcher
from pymomo.utilities.typedargs import *
from pymomo.exceptions import *

@context()
class OctopartMatcher (BOMMatcher):
	"""
	A service for pricing parts using the Octopart API.

	Currently parts can be matched using either a distributor's name and part number
	or directly using manufacturer's part numbers and the manufacturer's 
	name.  Realtime pricing information is returned as are any errors
	during the matching process. 
	"""

	URL = 'http://octopart.com/api/v3/'

	@param("key", "string", desc="Octopart API Key")
	def __init__(self, key=None):
		super(OctopartMatcher, self).__init__()

		if key is not None:
			self.api_key = key
		elif 'OCTOPART_KEY' in os.environ:
			self.api_key = os.environ['OCTOPART_KEY']
		else:
			raise ArgumentError('You need to either specify an Octopart API Key or have the OCTOPART_KEY environment variable set')

	def _build_call(self, method, args):
		"""
		Build a URL call with arg=val pairs including the apikey to the url specifying the method
		given.
		"""

		url = OctopartMatcher.URL + method + '?'

		out_args = []
		out_args.extend(args)
		out_args.append(('apikey', self.api_key))
		out_args.append(('include[]', 'short_description'))
		out_strings = map(lambda x: urllib.quote(x[0])+'='+urllib.quote(x[1]), out_args)

		query_string = "&".join(out_strings)

		return url + query_string

	def _build_query(self, part):
		query = {}
		if "distpn" in part:
			query['sku'] = part["distpn"]
		elif "mpn" in part:
			query['mpn'] = part["mpn"]
		else:
			raise ArgumentError("Octopart API cannot match a part without either a digikey part number or a manufacturer's part number", part=part)
		
		query['reference'] = part['ref']
		return query

	def _call(self, url):
		data = urllib.urlopen(url).read()
		response = json.loads(data)

		return response

	def _call_method(self, method, args):
		url = self._build_call(method, args)
		return self._call(url)

	def _build_offer(self, resp):
		"""
		Create a part_offer from an octopart api response
		"""

		offer = {}
		offer['seller'] = resp['seller']['name']
		offer['moq'] = resp['moq']
		offer['stock_quantity'] = resp['in_stock_quantity']
		offer['packaging'] = resp['packaging']
		offer['breaks'] = self._build_breaks(resp['prices'])

		return type_system.convert_to_type(offer, 'part_offer')

	def _build_breaks(self, resp):
		"""
		Create a price_list object from an octopart API response
		"""

		if 'USD' in resp:
			return type_system.convert_to_type(resp['USD'], 'price_list')

		return type_system.convert_to_type([], 'price_list')

	def _build_part(self, response):
		"""
		Create a physical_part object from an octopart api response
		"""

		part = {}

		manuname = ""
		if 'name' in response['manufacturer']:
			manuname = response['manufacturer']['name']

		part['manu'] = manuname
		part['mpn'] = response['mpn']
		
		part['desc'] = response['short_description']
 
		offers = map(lambda x: self._build_offer(x), response['offers'])
		part['offers'] = offers
		
		return type_system.convert_to_type(part, 'physical_part')

	def _match(self):
		for i in xrange(0, len(self.requests), 20):
			req = self.requests[i:i+20]
			queries = map(lambda x: self._build_query(x), req)

			query = json.dumps(queries)

			resp = self._call_method('parts/match',[('queries',query), ('exact_only', 'true')])

			for result in resp['results']:
				ref = result['reference']

				if len(result['items']) == 0:
					self.errors[ref] = {'msg': 'Did not match'}
					continue

				parts = map(lambda x: self._build_part(x), result['items'])
				self.matched[ref] = parts



#patch_mib12.py

import SCons.Builder

_patch_mib12 = SCons.Builder.Builder(
	action = 'patch_mib12_api.py $MIB_API_BASE $SOURCES $TARGET'
	)

def generate(env):
	env['BUILDERS']['patch_mib12'] = _patch_mib12

def exists(env):
	return 1


#merge.py
#Given two intelhex16bit objects, merge them together with various options

import intelhex

def insert_hex(stub, payload, start_addr, skip_ranges, payload_size):
	addrs = clip_payload(payload, payload_size)

	print "Inserting Hex file [0x%X, 0x%X]" % (addrs[0], addrs[-1])

	curr = start_addr

	if addrs[0] != 0:
		raise ValueError("Payload does not start at address 0.  This is probably not what you meant.")
		
	for i in xrange(addrs[0], addrs[-1]+1):
		curr, inc = clip_ranges(curr, skip_ranges)
		stub[curr] = payload[i]

		curr += 1

def clip_payload(payload, max_size):
	valid_addr = map(lambda x:x/2, filter(lambda x: x%2==0, payload.addresses())) #addresses are reported in bytes rather than words

	if max_size != 0:
		return filter(lambda x: x<max_size, valid_addr)

	return valid_addr

def clip_ranges(addr, ranges):
	"""
	If addr is contained in ranges, which must not be sequential,
	return the first valid address after the range containing addr
	otherwise just return addr unmodified. 
	"""
	for r,fun in ranges:
		if fun(addr):
			return r[1]+1, False

	return addr, True

def extract_hex(stub, start_addr, skip_ranges, payload_size):
	"""
	Given an IntelHex16bit object in stub, extract the contents starting at
	address start_addr and continuing sequentially for a maximum of payload_size
	bytes.  Do not extract the contents of skip_ranges.  The extracted hex file
	will be sequential before and after each skip range, i.e. those addresses 
	will be collapsed to that the last address before the range and the first
	address after the range are adjacent in the extracted file.
	"""

	maxsize = start_addr+payload_size
	if payload_size == 0:
		maxsize = 0

	addrs = clip_payload(stub, maxsize)
	addrs = filter(lambda x: x>=start_addr, addrs)
	addrs = filter(lambda x: clip_ranges(x, skip_ranges)[1]==True, addrs)

	curr = start_addr
	out = intelhex.IntelHex16bit()
	out.fill = 0x3FFF

	for i in xrange(0, len(addrs)):
		out[i] = stub[addrs[i]]

	return out


#ldf_compiler.py
#SCons Builder Action for creating header files from Log Definition Files

import SCons.Builder
import SCons.Action
import os.path
import sys
import utilities

sys.path.append(os.path.join(os.path.dirname(__file__), '..', '..'))
from pymomo.utilities.paths import convert_path

def ldf_generator(source, target, env, for_signature):
	"""
	Create an command line to drive the LDF compiler using the parameter defined in 
	the environment
	"""

	chip = env['ARCH']
	types = chip.property('type_package', None)

	#If the user has a custom type package import it so that the log file can find all of the
	#types that it needs to process the ldf file into a header.
	if types is None:
		args = ['momo --norc SystemLog LogDefinitionMap']
	else:
		args = ['momo --norc import_types "%s" SystemLog LogDefinitionMap' % types]

	for src in source:
		args.append('add_ldf \"%s\"' % str(src))

	args.append('generate_header \"%s\"' % str(target[0]))
	return SCons.Action.Action(" ".join(args), 'Compiling Log Definitions into %s' % str(target[0]))

_ldf_obj = SCons.Builder.Builder(
	generator = ldf_generator,
	suffix='.h')

def generate(env):
	env['BUILDERS']['ldf_compiler'] = _ldf_obj

def exists(env):
	return 1


#TODO:
# - add compare_descriptions function that compares the descriptions
#	currently associated with each bom line with those returned from
#	an online database side by side so that you can make sure that 
#	you're matching the right part.
# - Make updating metadata and attributes reflected realtime in board
#	without reloading the board.

from pymomo.utilities.typedargs import *
from pymomo.exceptions import *
from eagle.board import EagleBoard
from pricing import OfferRequirements
import match_engines
from bom_matcher import BOMMatcher
from decimal import Decimal
from production import ProductionFileGenerator
import itertools

#List of all of the types of board file we know how to deal with
board_types = {}
board_types['eagle'] = EagleBoard

@context()
class CircuitBoard:
	"""
	A CircuitBoard that can be used to create a BOM or produce gerbers

	The price of the BOM can be looked up using the Octopart API and 
	complex rules can be specifed to decide who to buy the parts from.
	A CircuitBoard can be created from an annotated pcb CAD file from any
	supported CAD program.  Currently we support EAGLE only but more will
	be added in the future.
	"""

	DefaultEngine = match_engines.OctopartMatcher

	@param("file", "path", "readable", desc="PCB board file to load")
	@param("type", "string", ("list", "eagle"), desc="type of file [eagle is only currently supported option]")
	def __init__(self, file, type='eagle'):
		self.board = board_types[type](file)
		self.clear_pricemodel()

		self._match_engine = CircuitBoard.DefaultEngine

		self.errors = []
		self.warnings = []

		#Process board into variants so that each variant is a list of unique parts
		self.variants = {name: self._process_variant(parts) for name,parts in self.board.data['variants'].iteritems()}

		#Copy all of the attributes from the board into this object, logging errors for any missing data
		self._set_attribute('company', self.board.data)
		self._set_attribute('part', self.board.data)
		self._set_attribute('width', self.board.data)
		self._set_attribute('height', self.board.data)
		self._set_attribute('units', self.board.data)
		self._set_attribute('revision', self.board.data)
		self._set_attribute('no_populate', self.board.data, default=[])
		self._set_attribute('unknown_parts', self.board.data, default=[])
		self._set_attribute('fab_template', self.board.data)
		self._set_attribute('fab_engine', self.board.data)

		#Make sure there are no parts that we don't have information about
		for x in self.unknown_parts:
			self._add_error(x, "Does not have required part information like distributor or manufacturer part numbers")

		#Make sure all parts have enough information to be matchable and placeable
		for variant, lines in self.variants.iteritems():
			for line in lines:
				for part in line:
					if not part.matchable():
						self._add_warning(part.name, "Does not have enough information to be matchable in variant %s" % variant)
					if not part.placeable():
						self._add_warning(part.name, "Does not have enough information to be placeble in variant %s" % variant)

		#Build a set containing all of the parts in all variants so we know what components could be populated but should not be
		self.all_parts = frozenset([x.name for x in self._iterate_all_parts()])
		self._build_lookup_table()

	@return_type("list(string)")
	@param("variant", "string", desc="Assembly variant to consider")
	def nonpopulated_parts(self, variant=None):
		"""
		Fetch the reference identifiers of all parts not populated in this variant.

		The list of strings that is returned does not include parts that are not populated
		in any assembly variant since those are considered to be virtual parts that should
		be ignored.  They are listed in the instance attribute no_populate.
		"""
		parts = frozenset([x.name for x in self._iterate_parts(variant)])

		nopop = self.all_parts - parts
		return [x for x in nopop]

	@return_type("bool")
	def is_clean(self):
		"""
		Return true if there are no errors or warnings about this board.
		"""

		return len(self.errors) == 0 and len(self.warnings) == 0

	@return_type("logical_part")
	@param("reference", "string", "not_empty", desc="Reference identifier to find")
	@param("variant", "string", desc="Assembly variant to search")
	def find(self, reference, variant=None):
		"""
		Find a part on the board using its reference identifier.

		If the same reference identifier refers to different parts in different
		assembly variants, then you must specify which one you want by passing
		the optional variant name.
		"""

		if reference not in self.part_index:
			raise ArgumentError("reference id did not exist", reference=reference)

		matches = self.part_index[reference]
		if len(matches) > 1 and variant is None:
			raise ArgumentError("reference id exists in multiple assembly variants, you must specify one", 
								reference=reference, variants=matches.keys())

		if len(matches) > 1:
			if variant in matches:
				return matches[variant]
			else:
				raise ArgumentError("reference id is not populated in specified assembly variant",
					reference=reference, variants=matches.keys(), specified_variant=variant)

		return matches.values()[0]

	@return_type("list(physical_part)")
	@param("reference", "string", "not_empty", desc="Reference identifier to find")
	@param("variant", "string", desc="Assembly variant to search")
	def lookup(self, reference, variant=None):
		"""
		Use the currently selected matching system to look up a component.

		The data returned can contain realtime price and stock information,
		a list of distributors that carry the part and other details depending
		on the matching system used.
		"""

		matcher = self._get_matcher()
		part = self.find(reference, variant)

		matcher.add_part(part)
		matcher.match_all()

		if not matcher.is_matched(part.name):
			raise DataError("part could not be matched")

		return matcher.match_info(part.name)

	@return_type("bool")
	@param("variant", "string", desc="Assembly variant to search")
	def match_status(self, variant=None):
		"""
		Return whether all parts are matched with a component database
		"""

		results = {}
		matcher = self._get_matcher()

		for part,num in self._iterate_lines(variant):
			if not part.matchable():
				return False

			matcher.add_part(part)

		matcher.match_all()		
		return matcher.all_matched()

	@return_type("map(string, string)")
	@param("variant", "string", desc="Assembly variant to search")
	def match_details(self, variant=None):
		"""
		Attempt to match all BOM components with a component database.

		Return a dictionary mapping reference numbers to matching statuses.
		This function will tell you if any of your parts were not successfully
		matched or did not have enough metadata to match to a unique component.
		"""

		results = {}
		matcher = self._get_matcher()

		for part,num in self._iterate_lines(variant):
			if not part.matchable():
				results[part.name] = 'Insufficient metadata to match'
				continue

			matcher.add_part(part)

		matcher.match_all()
		match_results = matcher.match_details()

		return dict(itertools.chain(results.iteritems(), match_results.iteritems()))

	@param("variant", "string", desc="Assembly variant to update")
	@param("missing_only", "bool", desc="Only update missing metadata attributes")
	def update_all_metadata(self, variant=None, missing=True):
		"""
		Update the entire board with additional metadata from the matching engine.

		This can be used to automatically fill in descriptions, manufacturers and mpns
		if you only have a distributor's part number, for example.  If missing is True,
		the default, only missing data will be added, entered data will not be overwritten.

		If variant is None, all assembly variants are updated.  If variant is passed, then
		only a single variant is updated.
		"""

		if isinstance(variant, basestring):
			update_vars = [self._assure_valid_variant(variant)]
		else:
			update_vars = self.variants.keys()

		handle = self.board.start_update()
		
		for var in update_vars:
			for part in self._iterate_parts(var):
				self.update_metadata(part.name, var, missing, handle=handle)

		self.board.finish_update(handle)

	@param("reference", "string", "not_empty", desc="Reference identifier to find")
	@param("variant", "string", desc="Assembly variant to search")
	@param("missing_only", "bool", desc="Only update missing metadata attributes")
	def update_metadata(self, reference, variant=None, missing=True, handle=None):
		"""
		Update a single part with additional metadata from the matching engine.

		This can be used to automatically fill in descriptions, manufacturers and mpns
		if you only have a distributor's part number, for example.  If missing is True,
		the default, only missing data will be added, entered data will not be overwritten.
		"""

		part = self.find(reference, variant)
		matches = self.lookup(reference, variant)
		if len(matches) > 1:
			raise DataError("part matched multiple times in database", reference=reference, variant=variant, matches=matches)
		
		match = matches[0]

		if match.mpn is not None and (not missing or (missing and part.mpn is None)):
			self.board.set_metadata(part, variant, 'MPN', match.mpn, handle=handle)

		if match.manu is not None and (not missing or (missing and part.manu is None)):
			self.board.set_metadata(part, variant, 'MANU', match.manu, handle=handle)

		if match.desc is not None and (not missing or (missing and part.desc is None)):
			self.board.set_metadata(part, variant, 'DESCRIPTION', match.desc, handle=handle)

	@param("output", "path", desc="Output directory for production files")
	def generate_fab(self, output):
		"""
		Create Gerber,excellon and readme files so this board can be fabricated

		Files are saved into the specified directory output, which is created if
		it does not exist.  A readme.txt file is also generated describing the
		contents of each file.
		"""

		prod = ProductionFileGenerator(self)
		prod.build_fab(output)

	@param("output", "path", desc="Output directory for production files")
	@param("varaint", "string", desc="Assembly variant to process")
	def generate_production(self, output, variant=None):
		"""
		Create Gerber files, BOMs and assembly drawings for this board.
		"""

		variant = self._assure_valid_variant(variant)

		prod = ProductionFileGenerator(self)
		prod.build_production(variant, output)

	@return_type("list(string)")
	def get_errors(self):
		"""
		Return a list of all errors in this board file.
		"""

		return map(self._format_msg, self.errors)

	@return_type("list(string)")
	def get_warnings(self):
		"""
		Return a list of all warnings in this board file.
		"""

		return map(self._format_msg, self.warnings)


	@param('dist', 'string', 'not_empty', desc='a distributor\'s name')
	def require_distributor(self, dist):
		"""
		When searching for prices, require one of these distributors

		This can be called multiple times to whitelist a number of 
		distributors.
		"""

		self.required_dists.append(dist)

	@param('dist', 'string', 'not_empty', desc='a distributor\'s name')
	def exclude_distributor(self, dist):
		"""
		When searching for prices, exclude offerings from this distributor

		This can be called multiple times to blacklist a number of distributors.
		"""

		self.excluded_dists.append(dist)

	@param('packaging', 'string', 'not_empty', desc='a type of packaging')
	def require_packaging(self, packaging):
		"""
		When searching for prices, require a certain type of packaging

		This can be called multiple times to whitelist a number of 
		packaging types like 'Cut Tape' or 'Tape & Reel'.
		"""

		self.required_packages.append(packaging)
	
	@param('packaging', 'string', 'not_empty', desc='a type of packaging')
	def exclude_packaging(self, packaging):
		"""
		When searching for prices, forbid a certain type of packaging

		This can be called multiple times to blacklist a number of 
		packaging types like 'Cut Tape' or 'Tape & Reel'.
		"""

		self.excluded_packages.append(packaging)

	@param("in_stock", "bool", desc="whether parts must be in stock")
	def require_stock(self, in_stock):
		"""
		Set whether parts must be in stock when pricing
		"""

		self.require_stock = in_stock

	@return_type("unit_price")
	@param("ref", "string", desc="part to lookup price information on")
	@param("n", "integer", "nonnegative", desc="number of parts to quote")
	@param("variant", "string", desc="Assembly variant to search")
	@param("total", "bool", desc="Return the total or unit price")
	def price(self, ref, n, variant=None, total=False):
		"""
		Return the best price of n of the part specified by the reference id, ref

		The part prices are searched according to the currently set price model
		including whitelisted and blacklisted packaging types and distributors.
		To find out information about what types of distributors and packaging a
		part is available in, use lookup.
		"""

		part = self.find(ref, variant)
		matcher = self._get_matcher()
		matcher.add_part(part)
		matcher.match_all()

		result = matcher.price_advanced(ref, n, self._build_pricemodel())

		if 'error' in result:
			raise DataError("Could not get price", error=error, result=result)

		if total:
			result['price'] *= n

		return type_system.convert_to_type(result, 'unit_price')

	@return_type("map(string, unit_price)")
	@param("n", "integer", "nonnegative", desc="number of complete boards to quote")
	@param("variant", "string", desc="Assembly variant to price")
	@param("excess", "integer", "nonnegative", desc="percent excess to order of each part")
	def detailed_prices(self, n, excess, variant=None):
		"""
		Return the price and seller for each part that minimizes cost for n boards 

		Optionally order excess percent extra of each part to account for assembly
		losses.
		"""

		quote_quant = n + (excess/100.0)

		out_prices = {line[0].desc: self.price(line[0].name, int(quote_quant*line[1] + 0.5), variant, total=True) for line in self._iterate_lines(variant)}
		return out_prices

	@return_type("map(string, price)")
	@param("n", "integer", "nonnegative", desc="number of complete boards to quote")
	@param("variant", "string", desc="Assembly variant to price")
	@param("excess", "integer", "nonnegative", desc="percent excess to order of each part")
	def total_price(self, n, excess, variant=None):
		"""
		Return the total cost and unit costs of n boards with excess percent excess

		The returned list contains 2 items, the first is the total price, the second
		"""

		prices = self.detailed_prices(n, excess, variant)

		total = Decimal('0.0')

		for price in prices.itervalues():
			total += price.price

		return {'total': total, 'unit': total/n}

	@annotated
	def clear_pricemodel(self):
		"""
		Remove all pricing restrictions
		"""

		self.required_dists = []
		self.excluded_dists = []
		self.required_packages = []
		self.excluded_packages = []
		self.prices = False
		self.require_stock = False
		self.quote_n = 1
		self.quote_quant = 1.0
		self.quote_excess = 0

	@param("quantity", "integer", "positive", desc="number of units to quote")
	@param("excess", "integer", "nonnegative", desc="required excess in %")
	def include_prices(self, quantity, excess):
		"""
		Include prices in generated BOMs
		"""

		self.quote_n = quantity
		self.quote_excess = excess
		self.quote_quant = quantity * (1.0 + excess/100.)
		self.prices = True

	@param("engine", "string", desc="name of component lookup engine to use")
	def set_match_engine(self, engine):
		"""
		Select the service used to lookup components.

		Currently supported values are CacheOnlyMatcher and OctopartMatcher
		to use only cached values or the Octopart API.  Other methods will be 
		supported in the future.
		"""

		if not hasattr(match_engines, engine):
			raise ArgumentError("unknown component matching engine specified", engine=engine)

		matcher = getattr(match_engines, engine)
		if not issubclass(matcher, BOMMatcher):
			raise ArgumentError("invalid component matching engine specified", engine=engine, classname=matcher, object=matcher)

		self._match_engine = matcher

	@return_type("list(bom_line)")
	@param("variant", "string", desc='assembly variant to build')
	def bom_lines(self, variant=None):
		"""
		Get all of the BOM lines for the specified assembly variant
		"""

		variant = self._assure_valid_variant(variant)

		outlines = []
		for line in self._iterate_linegroups(variant):
			outline = type_system.convert_to_type(line, 'bom_line')
			outlines.append(outline)

		return self._order_bom_lines(outlines)

	@return_type("list(string)")
	def get_variants(self):
		return self.variants.keys()

	@param("path", "path", "writeable", desc="The path to the BOM that should be created")
	@param("variant", "string", desc='assembly variant to build')
	@param("format", "string", ("list", ['excel']), desc="The file format that should be saved")
	@param("hide_problems", "bool", desc="do not include errors or warnings processing this file")
	def export_bom(self, path, variant=None, format="excel", hide_problems=False):
		"""
		Expore a Bill of Materials for this board in the specified format.

		BOMs can either be formatted as an industry standard excel spreadsheet
		or using a custom Cheetah based template. Currently only excel spreadsheets
		are supported.  The optional format parameter chooses the format and the 
		result is saved to a file at path.
		"""

		if format == 'excel':
			self._save_excel_bom(variant, path, hide_problems=hide_problems)

	def _save_excel_bom(self, variant, path, hide_problems):
		"""
		Save a BOM in excel format.
		"""

		import xlsxwriter
		import reference

		lib = reference.PCBReferenceLibrary()
		lines = self.bom_lines(variant)

		bk = xlsxwriter.Workbook(path)
		sht = bk.add_worksheet("Bill of Materials")
		
		#formats
		large = bk.add_format({'font_size': 16, 'bold':True})
		med   = bk.add_format({'font_size': 14, 'bold':True})
		bold  = bk.add_format({'bold': True})
		red   = bk.add_format({'bg_color': 'red'})
		orange= bk.add_format({'bg_color': 'orange'})

		#Write header
		sht.write(0, 0, "Bill of Materials", large)
		sht.write(1, 0, self.part, med)
		sht.write(2, 0, self.company)
		sht.write(3, 0, "Revision")
		sht.write(3, 1, self.revision)
		sht.write(4, 0, "Width")
		sht.write(4, 1, self.width)
		sht.write(4, 2, self.units)
		sht.write(5, 0, "Height")
		sht.write(5, 1, self.height)
		sht.write(5, 2, self.units)

		#Write out all of the BOM lines
		headers = ['', 'Qty', 'References', 'Value', 'Footprint', 'Description', 'Manufacturer', 'Part Number', 'Distributor', 'Dist. Part Number']
		row = 7
		col = 0

		for h in headers:
			sht.write(row, col, h, bold)
			col += 1

		row += 1
		for i,line in enumerate(lines):
			sht.write(row, 0, i+1)
			sht.write(row, 1, line.count)
			sht.write(row, 2, ", ".join(line.refs))

			if lib.has_value(line.type):
				sht.write(row, 3, line.value)

			sht.write(row, 4, line.package.name)
			sht.write(row, 5, line.desc)

			if line.manu is not None:
				sht.write(row, 6, line.manu)
			if line.mpn is not None:
				sht.write(row, 7, line.mpn)
			if line.dist is not None:
				sht.write(row, 8, line.dist)
			if line.distpn is not None:
				sht.write(row, 9, line.distpn)

			row += 1

		row += 1

		#Write out all of the unpopulated parts
		sht.write(row, 0, "Unpopulatd Parts", red)
		sht.write(row, 1, ", ".join(self.nonpopulated_parts(variant)), red)

		if not hide_problems:
			row += 2
			if len(self.errors) > 0:
				errors = self.get_errors()
				sht.write(row, 0, "BOM Errors", bold)
				row += 1
				for error in errors:
					sht.write(row, 0, error)
					row += 1

			row += 1
			if len(self.warnings) > 0:
				warnings = self.get_warnings()
				sht.write(row, 0, "BOM Warnings", bold)
				row += 1
				for warning in warnings:
					sht.write(row, 0, warning)
					row += 1

		bk.close()

	def _order_bom_lines(self, lines):
		"""
		Sort bom lines based on the type of each line

		Follow a standard order: R, C, L, D, U, other
		This routine assumes that there will never be more than 100,000
		parts per board, which seems safe for the foreseeable future.
		"""

		order_dict = {'R': 0, 'C': 1, 'L': 2, 'D': 3, 'U': 4}

		def ref_key(line):
			mult = 100000
			ref = line.type
			num = line.lowest
			if ref in order_dict:
				return order_dict[ref]*mult + num

			if ref in ref_key.others:
				return ref_key.others[ref]*mult + num

			ind = ref_key.other_index
			ref_key.others[ref] = ref_key.other_index
			ref_key.other_index += 1
			return ind*mult + num

		ref_key.other_index = 20
		ref_key.others = {}

		return sorted(lines, key=ref_key)

	def _build_pricemodel(self):
		return OfferRequirements(self.required_dists, self.excluded_dists,
								 self.required_packages, self.excluded_packages, 
								 self.require_stock)

	def _default_variant(self):
		"""
		If there is only one variant, choose it by default
		"""

		variants = self.variants.keys()
		if len(variants) == 1:
			return variants[0]

		return None

	def _assure_valid_variant(self, variant):
		"""
		If variant is None, attempt to use the default variant.
		"""

		if variant is None:
			variant = self._default_variant()
			if variant is None:
				raise ArgumentError('you must specify a variant when there is more than one option', variants=self.board.variants.keys())

		return variant

	def _process_variant(self, parts):
		"""
		Group items by unique key and return tuples of identical parts so that you can 
		turn a list of parts into a BOM with identical line items.
		"""

		bomitems = []

		sparts = sorted(parts, key=lambda x:x.unique_id())
		for k,g in itertools.groupby(sparts, lambda x:x.unique_id()):
			bomitems.append(list(g))

		return bomitems

	def _add_warning(self, attribute, msg):
		"""
		Log a warning about something missing or not correct about this circuit board.
		"""

		self.warnings.append(('WARNING', attribute, msg))

	def _add_error(self, attribute, msg):
		"""
		Log an error about something missing or not correct about this circuit board.
		"""

		self.errors.append(('ERROR', attribute, msg))

	def _format_msg(self, msg_tuple):
		"""
		Format an error or warning message into a string
		"""

		return "%s on attribute/part %s: %s" % msg_tuple

	def _set_attribute(self, attribute, data, default="Unknown Attribute", required=True):
		value = default

		if required:
			logger = self._add_error
		else:
			logger = self._add_warning

		if attribute not in data:
			logger(attribute, "Board data did not contain attribute in dictionary")
		else:
			brd_value = data[attribute]
			if brd_value is None:
				logger(attribute, "Attribute was empty or not defined")
			else:
				value = brd_value

		setattr(self, attribute, value)

	def _build_lookup_table(self):
		"""
		Build a map to look up part information given its reference number
		"""

		part_table = {}

		for variant, lines in self.variants.iteritems():
			for line in lines:
				for part in line:
					if part.name not in part_table:
						part_table[part.name] = {}
					
					part_table[part.name][variant] = part

		self.part_index = part_table

	def _iterate_parts(self, variant=None):
		"""
		Iterate over all parts in the assembly variant specified
		"""

		variant = self._assure_valid_variant(variant)
		lines = self.variants[variant]

		for line in lines:
				for part in line:
					yield part

	def _iterate_lines(self, variant=None):
		"""
		Iterate over each distinct BOM line on the board returning
		the first logical part and the number of identical parts
		"""

		variant = self._assure_valid_variant(variant)
		lines = self.variants[variant]

		for line in lines:
			yield line[0], len(line)

	def _iterate_linegroups(self, variant=None):
		"""
		Iterate over all parts grouping them by their bom line

		This returns lists of parts corresponding to each BOM line
		"""

		variant = self._assure_valid_variant(variant)
		lines = self.variants[variant]

		for line in lines:
			yield line

	def _iterate_all_parts(self):
		"""
		Iterate over all parts defined in any assembly variant of this board
		"""

		for variant, lines in self.variants.iteritems():
			for line in lines:
				for part in line:
					yield part

	def _get_matcher(self):
		"""
		Get an instance of the currently supported match engine
		"""

		return self._match_engine()




#board.py
#TODO
# - check for location of eagle in standard locations on Windows

import itertools
import xml.etree.cElementTree as ElementTree

import platform
import os
import subprocess

from part import part_from_board_element
from ..reference import PCBReferenceLibrary
from ..package import Package
from pymomo.exceptions import *

class EagleBoard:
	def __init__(self, brd_file):
		"""
		Create a Board from EAGLE Board file by parsing attribute tags embedded with the
		components.  Create different assembly variants for each different configuration
		specified in the file
		"""
		
		tree = ElementTree.parse(brd_file)
		if tree is None:
			raise DataError("Eagle board file does not contain valid XML", file=brd_file)

		root = tree.getroot()

		#Find all of the defined packages in this file
		packages =root.findall('.//packages')

		elems = root.find("./drawing/board/elements")
		if elems is None:
			raise DataError("Eagle board file does not contain a list of components", file=brd_file)

		parts = list(elems)

		unknown = []
		nopop = []
		constant = []
		variable = []
		variants = find_variants(root)

		all_vars = set(variants)

		#Create a part object for each part that has a valid digikey-pn
		#If there are multiple digikey part numbers, create a part for each one
		for part in parts:			
			valid_part = False
			found_vars = set()
			nopop_vars = set()

			for v in variants:
				p,found = part_from_board_element(part, v, packages)
				if p and found:
					variable.append( (v, p) )
					found_vars.add(v)
					valid_part = True
				elif found:
					#If this part has digipn == "" or Populate=no, then it should not be populated in this variant
					valid_part = True
					nopop_vars.add(v)
					nopop.append(part.get('name',"Unknown Name"))

			#See if this is a constant part (with no changes in different variants)
			p,found = part_from_board_element(part, "", packages)
			if p and len(found_vars) == 0 and len(nopop_vars) == 0:
				constant.append(p)
				valid_part = True
			elif p and len(found_vars) != len(all_vars):
				#there are some variants that want the default part and some that want a change
				for var in (all_vars - found_vars) - nopop_vars:
					variable.append( (var, p) )
					valid_part = True

			#Make sure this part has information for at least one assembly variant
			if not valid_part:
				unknown.append(part.get('name',"Unknown Name"))

		vars = {}
		#Create multiple variants
		for var in variants:
			vars[var] = constant + filter_sublists(variable, var)

		props = {}
		props['company'] = find_attribute(root, 'COMPANY', None)
		props['part'] = find_attribute(root, 'PARTNAME', None)
		props['width'] = find_attribute(root, 'WIDTH', None)
		props['height'] = find_attribute(root, 'HEIGHT', None)
		props['units'] = find_attribute(root, 'DIM_UNITS', None)
		props['revision'] = find_attribute(root, 'REVISION', None)
		props['no_populate'] = nopop
		props['unknown_parts'] = unknown
		props['variants'] = vars
		props['fab_engine'] = 'eagle' #Used by production.py to locate the templates for gerber generation
		props['fab_template'] = find_attribute(root, 'FAB_TEMPLATE', None)
		
		self.data = props
		self.file = brd_file

	def start_update(self):
		"""
		Open the underlying file for appending additional attributes

		Returns an opaque datatype that should be passed to set_metadata.
		"""

		tree = ElementTree.parse(self.file)
		if tree is None:
			raise DataError("Eagle board file does not contain valid XML", file=brd_file)

		return tree

	def finish_update(self, handle):
		"""
		Save the changes to the board file specified in handle.

		handle must be the result of a call to start_update
		"""

		handle.write(self.file)		

	def set_metadata(self, part, variant, key, value, handle=None):
		"""
		Update this board file with additional attributes.

		The board must be reloaded for the attributes to be visible.
		"""

		if handle is None:
			tree = self.start_update()
			root = tree.getroot()
		else:
			root = handle.getroot()

		element = root.find(".//element[@name='%s']" % part.name)
		if element is None:
			raise ArgumentError("Part was not found in board to update")

		att_name = key.upper()

		if att_name not in set(['MANU', 'MPN', 'DIST', 'DIST-PN', 'DIGIKEY-PN', 'FOOTPRINT', 'DESCRIPTION']):
			raise ArgumentError("attempting to set unknown metadata attribute", file=self.file, attribute=att_name, value=value, variant=variant)

		if variant is not None and variant != 'MAIN':
			att_name += '-%s' % variant.upper()

		att_elem = element.find("./attribute[@name='%s']" % att_name)
		if att_elem is not None:
			att_elem.set('value', value.upper())
		else:
			x = element.get('x', '0.0')
			y = element.get('y', '0.0')

			#Add in other default attributes for this xml element.
			attribs = {	'x': x, 'y': y, 'name': att_name, 'value': value.upper(),
						'size': "1.778", 'layer': "27", 'display': "off"}

			ElementTree.SubElement(element, 'attribute', **attribs)

		#Update the file with the modification if we were called in standalone mode
		if handle is None:
			tree.write(self.file)

	def build_production_file(self, path, layers, type, **kwargs):
		"""
		Use the data in this board file to produce Gerbers and Excellon files

		layers should be the eagle layers to include in the file
		type should be gerber, excellon or drawing
		optional kwargs may be supported in the future to adjust output
		"""

		#Options from EAGLE's built in gerber file for producing 2 layer boards
		#and argument names from EAGLE -? command
		args = ['-X', '-f+', '-c+', '-O+']

		if type == 'gerber':
			args.append('-dGERBER_RS274X')
		elif type == 'excellon':
			args.append('-dEXCELLON')
		elif type == 'drawing':
			args.append('-dPS')
			args.append('-h11')
			args.append('-w7.75')
			args.append('-s2.5')
		else:
			raise ArgumentError("Invalid type specified for production file (must be gerber, excellon or drawing)",  type=type)

		args.append('-o%s' % path)
		args.append(self.file)

		for layer in layers:
			args.append(layer)

		self._execute_eagle(args)

	def _find_eagle(self):
		"""
		Try to find the eagle executable on this system.

		If it's in the path, just return that, otherwise search in standard
		locations on each architecture or fail.
		"""

		from distutils.spawn import find_executable

		if platform.system() == 'Darwin':
			eagle = find_executable('EAGLE')
			if eagle is not None:
				return eagle

			entries = os.listdir('/Applications')
			possible_eagles = filter(lambda x: x.startswith('EAGLE-') and os.path.isdir(os.path.join('/Applications', x)), entries)

			#Try to get the latest EAGLE version if there are multiple installed
			try:
				possible_eagles.sort(key=lambda x: map(int, x[6:].split('.')))

				eagle_dir = possible_eagles[-1]
				eagle_path = os.path.join('/Applications', eagle_dir, 'EAGLE.app', 	'Contents', 'MacOS', 'EAGLE')

				if os.path.isfile(eagle_path) and os.access(eagle_path, os.X_OK):
					return eagle_path
			except ValueError:
				#Value errors mean we could not convert the version string to an int, indicating that it wasn't in the form X.Y.Z
				#ignore this error and just say we couldn't find EAGLE since an eagle directory without an X.Y.Z form could be a
				#different program altogether.
				pass
		elif platform.system() == 'Windows':
			eagle = find_executable('EAGLECON')
			if eagle is not None:
				return eagle

			#TODO: Check for eagle location on windows and autofind
			#Check for program files directory in 32 or 64 bit mode like so:
			#http://stackoverflow.com/questions/1283664/python-get-wrong-value-for-os-environprogramfiles-on-64bit-vista
		else:
			#Otherwise assume that Eagle will be in $PATH
			eagle = find_executable('EAGLECON')
			if eagle is not None:
				return eagle

		raise EnvironmentError("could not find installed version of Cadsoft EAGLE", suggestion='install EAGLE and add it to your $PATH')

	def _execute_eagle(self, args):
		eagle = self._find_eagle()
		
		#TODO: Check what kinds of exceptions this can throw
		with open(os.devnull, 'wb') as DEVNULL:
			subprocess.check_call([eagle] + args, stdout=DEVNULL, stderr=DEVNULL)


#Helper functions
def remove_prefix(s, prefix):
	if not s.startswith(prefix):
		return s

	return s[len(prefix):]

def filter_sublists(master, key):
	"""
	Given a list of lists where each of the sublist elements are tuples (key, value),
	return a concatenated list of all values in tuples that match key
	"""

	concat = itertools.chain(master)

	filtered = filter(lambda x: x[0] == key, concat)

	return map(lambda x:x[1], filtered)

MISSING = object()
def find_attribute(root, name, default=MISSING):
	attr = root.find(".//attribute[@name='%s']" % name)
	if attr is None:
		if default is not MISSING:
			return default

		raise DataError("required board attribute not found", name=name)

	return attr.get('value')

def find_variants(root):
	vars = root.findall(".//attribute[@value='%s']" % 'ASSY-VARIANT')

	if len(vars) == 0:
		return ["MAIN"]
	else:
		return map(lambda x: x.get('name'), vars)



from logstatement import LogStatement

class LogHex (LogStatement):
	def __init__(self, statement, log):
		if len(statement['data']) != 1:
			raise ValueError('Invalid hex logging statement, too much data (len=%d)' % len(statement['data']))

		self.data = statement['data'][0]['value']
		self.pull_info(log)

	def format(self):
		return "Logged 0x%X" % (self.data)


#syslog package

"""
A package for interfacing with the momo hardware syslog.

Methods and objects for dealing with the persistent system log 
present on MoMo controller boards including parsing the log
and preparing log statement definitions for interpreting the
log.
"""

_name_ = "SystemLog"

#Outside accessible API for this package
from descriptor import LogDefinitionMap
from logentry import RawLogEntry
from syslog import SystemLog


from utils import *

class GotoInstruction:
	def __init__(self, args):
		self.args = parse_args("a", args)

	def __str__(self):
		return "goto 0x%x" % self.args[0]

	def encode(self):
		return (0b101 << 11) | self.args[0]


#unit_test.py

import os.path
import os
from SCons.Environment import Environment
import test_summary
import fnmatch
from pymomo.exceptions import *

known_types = {}

class UnitTest (object):
	def __init__(self, files, ignore_extra_attributes=False):
		self.files = files
		self.additional_intermediates = []
		self.extra_modules = []
		self.desc = ''
		self.targets = None
		self.patch = {}

		self.ignore_extra_attributes = ignore_extra_attributes
		self.basedir = os.path.dirname(files[0])

		self._check_files()
		self._extract_header(files[0])
		self._check_files()

		self.status = "Unknown"
	
	def show(self):
		print "#Unit Test#"
		print self.name

		if self.targets is None:
			print "Targets: All module targets"
		else:
			print "Targets:", ", ".join(self.targets)
		print "Status:", self.status
		print self.desc

	def build_dirs(self, chip):
		"""
		Return the appropriate build directories for this unit test.  The build hierarchy is:
		- test: build/test/<CHIP>/<TEST>
		- objects: build/test/<CHIP>/<TEST>/objects
		"""

		basedirs = chip.build_dirs()

		testdir = os.path.join(basedirs['test'], self.name, 'objects')
		outdir = os.path.join(basedirs['test'], self.name)
		finaldir = os.path.join('build', 'test', 'output')
		logdir = os.path.join(finaldir, 'logs')

		return {'test':outdir, 'objects': testdir, 'logs': logdir}

	def get_path(self, obj, chip):
		"""
		Get the canonical path for various build products of the unit test.  Currently understands:
		- rawlog: the raw output from the simulator run
		- log: the processed version of rawlog into a standard format
		"""

		dirs = self.build_dirs(chip)

		if obj == 'rawlog':
			name = self.name + '@' + chip.arch_name() + '.raw'
			return os.path.join(dirs['objects'], name)
		elif obj == 'log':
			name = self.name + '@' + chip.arch_name() + '.log'
			return os.path.join(dirs['logs'], name)
		elif obj == 'status':
			name = self.name + '@' + chip.arch_name() + '.status'
			return os.path.join(dirs['logs'], name)
		elif obj == 'elf':
			name = self.name + '.elf'
			return os.path.join(dirs['objects'], name)
		else:
			raise BuildError("Path to unknown build product asked for", unit_test=self.name, object=obj, arch=chip.name)

	def build_target(self, target, summary_env):
		raise BuildError('The build_target method must be overriden by a UnitTest subclass', unit_test=self.name, type=str(self.__class__))

	def check_output(self, chip, log):
		"""
		If unit tests have additional things to check about the unit test's output,
		they should override this function and return False if the test fails these
		additional checks. They may also choose to open log for appending and log the
		reason for this failure. 
		"""

		return True

	def save_status(self, path, passed):
		with open(path, "w") as f:
			if passed :
				f.write('PASSED')
			else:
				f.write('FAILED')

	def build(self, module_targets, summary_env):
		"""
		Build this unit test for the intersection of the targets that it is designed for
		and the targets that the module it is targeted at is designed for.  If more architectures
		are specified in the module_targets than in a unit test, the unit test is assumed to
		apply if all of the architectures in the unit test are contained in the module_target.
		"""

		for mod_target in module_targets:
			found = False
			if self.targets is not None:
				for target in self.targets:
					target_archs = set(target.split('/'))
					if target_archs <= mod_target.archs():
						found = True
						break

				if found is False:
					continue

			self.build_target(mod_target, summary_env)

	def _check_files(self):
		"""
		All files should be c or assembly files (.c or .as) and should
		exist.
		"""

		for file in self.files:
			valid = set (['.c', '.as', '.asm'])
			ext = os.path.splitext(file)[1]

			if not os.path.exists(file):
				raise BuildError('Test source file does not exist', file=file)
			if ext not in valid:
				raise BuildError('Test source file does not has .c,.as or .asm extension', file=os.path.basename(file))

		return True

	def _parse_targets(self, targets):
		"""
		Parse the targets specified for this unit test into canonical format
		allowing subclasses to handle targets specified for them by overriding
		the _parse_target(self, target) method to return the canonical name
		for the target
		"""

		if targets.lower() == 'all':
			self.targets = None
			return

		targets = targets.split(',')

		canonical_targets = []

		for target in targets:
			t = target.strip()
			canonical_targets.append(self._parse_target(t))

		self.targets = canonical_targets

	def find_support_file(self, support_name, target):
		"""
		Given the name of a support file, try to find it by looking in the 
		same directory as the first unit test file and searching the following
		names, where target is a list of architectures like:
		arch1/arch2/arch3
		
		name_arch1_arch2_arch3.ext
		name_arch1_arch2.ext
		name_arch1.ext
		name.ext

		If no file can be found, return None.
		"""

		name, ext = os.path.splitext(support_name)

		prefixes = list(reversed(target.arch_prefixes()))
		prefixes.append([])
		for prefix in prefixes:
			name_str = "_".join([name] + prefix)
			name_str += ext

			if os.path.exists(name_str):
				return name_str

		raise BuildError("could not find support file", filename=support_name, arch_prefixes=prefixes)

	def _extract_header(self, file):
		"""
		Process the first commented block in the file and extract the assignments
		from it, parsing out the name, targets, and description
		"""

		header = []
		with open(file, "r") as f:
			for line in f:
				line = line.strip()

				#ignore blank lines
				if line == '':
					continue

				if line[0] == ';':
					header.append(line[1:])
				elif line[0:2] == '//':
					header.append(line[2:])
				else:
					break

		for line in header:
			v = line.split(':')

			#Undelimited lines after the description are assumed
			#to be continuations of the description
			if len(v) < 2:
				if hasattr(self, 'desc'):
					self.desc += v[0]
			elif len(v) == 2:
				name = v[0]
				val = v[1]
				name = name.strip()
				val = val.strip()

				name = name.lower()
				
				#Check if we support this attribute natives
				if name == 'name':
					self.name = val
				elif name == 'type':
					self.type = val
				elif name == 'description':
					self.desc = val
				else:
					#All other attributes are parsed by handlers that either
					#we or a subclass provide, so check if the attribute has
					#a handler or fail

					name = name.replace(' ', '_') #make sure this is a valid python identifier name

					handlern = '_parse_%s' % name
					if hasattr(self, handlern):
						handler = getattr(self, handlern)
						handler(val)
					elif not self.ignore_extra_attributes: 
						raise BuildError("Unknown attribute encountered with no handler", file=file, attribute=name, value=val, type=str(self.__class__))

		#Make sure that all of the right information has been found
		required_attributes = ['name', 'type']
		for attr in required_attributes:
			if not hasattr(self, attr):
				raise BuildError("test does not have a complete header", file=file, missing_attribute=attr)

	def _parse_target(self, value):
		return value

	def _parse_patches(self, value):
		"""
		Parse a list of symbols that need to be patched in the test. Each specific architecture
		can choose to do what it wants with this list.
		"""

		parsed = value.split(',')
		pairs = [x.rstrip().lstrip().split('->') for x in parsed]
		mapper = {name: value for name,value in pairs}
		self.patch = mapper

	def add_intermediate(self, file, folder=None):
		"""
		Add a file to the list of intermediate build products produced by this unit test.
		This is important for letting SCons know about the file so that it can clean it up
		properly.  If folder is specified, is should be one of the values returned by
		build_dirs() and it will be replaced with the appropriate build directory for the
		target being built.
		"""

		self.additional_intermediates.append((folder, file))

	def get_intermediates(self, chip):
		dirs = self.build_dirs(chip)

		paths = []

		for folder, p in self.additional_intermediates:
			if folder is None:
				paths.append(p)
			 	continue

			basepath = dirs[folder]
			paths.append(os.path.join(basepath, p))

		return paths

def find_units(parent):
	files = [f for f in os.listdir(parent) if os.path.isfile(os.path.join(parent, f))]

	#ignore hidden files
	files = filter(lambda x: x[0] != '.', files)

	#ignore files that start with support_
	files = filter(lambda x: not x.startswith("support_"), files)

	files = [os.path.join(parent, f) for f in files]

	tests = []

	for f in files:
		unit = UnitTest([f], ignore_extra_attributes=True)
		
		if unit.type not in known_types:
			raise BuildError("Unknown test type in unit test", name=unit.name, type=unit.type)

		test = known_types[unit.type]([f])
		tests.append(test)

	return tests

def find_sources(src_dir, patterns=('*.c', '*.as', '*.asm')):
	"""
	Given a source directory, recursively find all source and header files under that directory
	"""

	include_dirs = set()
	src = {}
	headers = {}

	for root, dirnames, filenames in os.walk(src_dir):
		dir_headers = fnmatch.filter(filenames, '*.h')
		if len(dir_headers) > 0:
			include_dirs.add(root)

		headers.update({os.path.splitext(x)[0]: os.path.join(root, x) for x in dir_headers})

		for s in patterns:
			dir_srcs = fnmatch.filter(filenames, s)
			src.update({os.path.splitext(x)[0]: os.path.join(root, x) for x in dir_srcs})

	return include_dirs, src, headers


def build_units(parent, targets):
	tests = find_units(parent)

	summary_env = Environment()
	summary_env['TESTS'] = []

	for test in tests:
		test.build(targets, summary_env)

	build_summary(summary_env)

def build_summary_name():
	return os.path.join('build', 'test', 'output', 'results.txt')

def build_summary(env):
	env.Command([build_summary_name()], env['TESTS'], action=env.Action(test_summary.build_summary_cmd, "Creating test summary"))



import sys
import os.path
import os
from random import randint

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from pymomo.commander.meta import *
from pymomo.commander.exceptions import *
import cmdln

import sched, time

class MoMoSensor(cmdln.Cmdln):
	name = 'MoMoSensor'

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_emulate(self, subcmd, opts, sensor_address):
		"""${cmd_name}: Log a random (1-1000) sensor value every .1 seconds

		${cmd_usage}
		${cmd_option_list}
		"""
		
		con = self._get_controller(opts)

		meta = 0x0;
		s = sched.scheduler(time.time, time.sleep)
		def log_event( scheduler ):
			s.enter( 1, 1, log_event, (scheduler,) )
			value = randint(1,1000)
			con.sensor_log( int(sensor_address), int(meta), value )
			print "Logged event!  Value: %d" % value

		s.enter( 1, 1, log_event, (s,) )
		s.run()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_test(self, subcmd, opts):
		"""${cmd_name}: Test subsection bound overflow

		${cmd_usage}
		${cmd_option_list}
		"""
		
		con = self._get_controller(opts)

		meta = 0x0;
		s = sched.scheduler(time.time, time.sleep)
		def log_event( scheduler, count, values ):
			memory_subsection_count = 16 * 12 - 1
			events_per_subsection = 341 # 4096/12
			if count >= events_per_subsection*memory_subsection_count + 10: # = when to stop
				print "COMPLETE"
				debug = con.sensor_log_debug()
				print "DEBUG ADDRESSES: MIN %d, MAX %d, START %d, END %d" % con.sensor_log_debug()
				print "Try reading past the end..."
				while len(values) > 0: #should be 351 at this point, read past the end of the buffer to test wrapping.

					v = con.sensor_log_read().value
					ev = values.pop(0)
					if v != ev:
						print "FAILED: %d != %d (expected)" % ( v, ev )
						return
					else:
						print "(%d) read %d" % (len(values), v)
					
					if len(values) != con.sensor_log_count():
						print "FAILED"
						print "count: %d, expected: %d" % (con.sensor_log_count(), len(values))
						print "DEBUG ADDRESSES: MIN %d, MAX %d, START %d, END %d" % con.sensor_log_debug()
						return
				print "SUCCESS"
				print "DEBUG ADDRESSES: MIN %d, MAX %d, START %d, END %d" % con.sensor_log_debug()
				return

			if ( len(values) > events_per_subsection*memory_subsection_count ):
				for i in range(events_per_subsection):
					values.pop(0)
			if ( len(values) != con.sensor_log_count() ):
				print "FAILURE"
				print "count: %d, expected: %d" % (con.sensor_log_count(), len(values))
				print "DEBUG ADDRESSES: MIN %d, MAX %d, START %d, END %d" % con.sensor_log_debug()
				return

			value = randint(1,1000)
			values.append(value)
			con.sensor_log( 42, int(meta), value )
			print "(%d) Logged event!  Value: %d" % (len(values), value)
			s.enter( .01, 1, log_event, (scheduler,count+1,values) )

		con.sensor_log_clear()
		print "All sensor events cleared from PIC memory."
		values = []
		s.enter( .1, 1, log_event, (s,0,values) )
		s.run()

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_readone(self, subcmd, opts):
		"""${cmd_name}: Read one event from the log.

		${cmd_usage}
		${cmd_option_list}
		"""
		
		con = self._get_controller(opts)
		event = con.sensor_log_read()

		print "Stream: %d" % event.stream
		print "MetaData: %d" % event.metadata
		print "Timestamp: %s" % str( event.timestamp )
		print "Value: %d" % event.value

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_log(self, subcmd, opts, sensor_address, meta, value):
		"""${cmd_name}: Log one integer value (32-bit)

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller(opts)
		con.sensor_log( int(sensor_address), int(meta), int(value) )
		print "Logged event!"

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_count(self, subcmd, opts):
		"""${cmd_name}: Number of events in the sensor log.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller(opts)
		print con.sensor_log_count();

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_clear(self, subcmd, opts):
		"""${cmd_name}: Clear all values from the sensor log, resetting the pointers to
		  the beginning of the flash_queue.

		${cmd_usage}
		${cmd_option_list}
		"""
		con = self._get_controller(opts)
		con.sensor_log_clear();
		print "All sensor events cleared from PIC memory."

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	def do_debug(self, subcmd, opts):
		"""${cmd_name}: Print out the min, max, start, and end addresses

		${cmd_usage}
		${cmd_option_list}
		"""
		
		con = self._get_controller(opts)

		print "DEBUG ADDRESSES: MIN %d, MAX %d, START %d, END %d" % con.sensor_log_debug()

	def _get_controller(self, opts):
		try:
			c = get_controller(opts.port)
		except NoSerialConnectionException as e:
			print "Available serial ports:"
			if not e.available_ports():
				print "<none>"
			else:
				for port, desc, hwid in e.available_ports():
					print "\t%s (%s, %s)" % ( port, desc, hwid )
			self.error(str(e))
		except ValueError as e:
			self.error(str(e))

		return c

def main():
	emulator = MoMoSensor()
	return emulator.main()


#!/usr/bin/env python

import sys, os
from time import sleep

sys.path.append(os.path.join(os.path.dirname(__file__), '..'))

from pymomo.commander.meta import *
from pymomo.commander.exceptions import *
from pymomo.commander.proxy import *

import time
import cmdln

class SensorTool(cmdln.Cmdln):
	name = 'multisensor'

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the multisensor module' )
	@cmdln.option('-n', '--average', type=int, default=1, help='Average for N samples' )
	@cmdln.option('-c', '--continuous', action='store_true', default=False, help='Continually sample and report' )
	def do_read(self, subcmd, opts):
		"""${cmd_name}: Read the voltage from the multisensor module

		${cmd_usage}
		${cmd_option_list}
		"""

		sens = self._create_proxy(opts)

		while True:
			v = self._average(sens, opts.average)
			print "Voltage: %d" % v

			if not opts.continuous:
				break

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the multisensor module' )
	def do_set(self, subcmd, opts, name, value):
		"""${cmd_name}: Set the parameter 'name' to the value 'value'

		Valid parameter names are: offset, gain1, gain2, select, invert and delay
		Valid values are: offset: [0, 255], gain1: [0,127], gain2: [0,7], 
		select: [current|differential], invert: [yes|no], delay: [1, 255]

		${cmd_usage}
		${cmd_option_list}
		"""

		sens = self._create_proxy(opts)
		sens.set_parameter(name, value)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the multisensor module' )
	def do_set_push(self, subcmd, opts, enabled):
		"""${cmd_name}: Set whether sensor data is pushed to the get_controller

		Valid options are enabled or disabled.

		${cmd_usage}
		${cmd_option_list}
		"""

		sens = self._create_proxy(opts)
		
		if enabled.lower() == 'enabled':
			en = True
		elif enabled.lower() == 'disabled':
			en = False
		else:
			print "Invalid option: use enabled or disabled"
			return

		sens.set_sensor_push(en)

	@cmdln.option('-p', '--port', help='Serial port that fsu is plugged into')
	@cmdln.option('-a', '--address', help='The MIB address of the multisensor module' )
	@cmdln.option('-c', '--continuous', action='store', type=int, default=1, help='Continually sample and report' )
	def do_pulse(self, subcmd, opts):
		"""${cmd_name}: Sample the pulse counters for one sample period and return the pulses seen
	
		${cmd_usage}
		${cmd_option_list}
		"""

		sens = self._create_proxy(opts)

		while True:
			sens.acquire_pulses()
			
			time.sleep(0.6)
			pulses, invalid = sens.read_pulses()
			median = sens.read_median()
			med_pulses, invalid = sens.read_pulses()

			print "%d pulses (not including %d invalid pulses)" % (len(pulses), invalid)
			print "Median pulse interval: %.0f ms" % (median*8.e-3,)

			for width, interval in pulses:
				int_str = "---"
				if interval > 0:
					int_str = "%.0f ms" % (interval*8.e-3,)

				print "width: %.0f ms, spacing: %s" % (width*8.e-3, int_str)
			

			if not opts.continuous:
				break

			time.sleep(opts.continuous)

	def _create_proxy(self,opts):
		try:
			con = get_controller(opts.port)
			address = 11
			if opts.address != None:
				address = opts.address
			gsm = MultiSensorModule(con.stream, address)
		except NoSerialConnectionException as e:
			print "Available serial ports:"
			if not e.available_ports():
				print "<none>"
			else:
				for port, desc, hwid in e.available_ports():
					print "\t%s (%s, %s)" % ( port, desc, hwid )
			self.error(str(e))
		except ValueError as e:
			self.error(str(e))

		return gsm

	def _average(self, sens, n):
		readings = []
		for i in xrange(0, n):
			v = sens.read_voltage()
			readings.append(v)

		return sum(readings)/len(readings)

def main():
	sensortool = SensorTool()
	return sensortool.main()


#i2c.py

class I2CMasterSequence:
	start_delay = 100
	half_baud = 5

	def __init__(self, start):
		self.start = start
		self.scl = []
		self.sda = []
		self.comments = []
		self.cycle_count = start

	def _scl_state(self):
		if len(self.scl) == 0:
			return 1

		return self.scl[-1][1]

	def _sda_state(self):
		if len(self.sda) == 0:
			return 1

		return self.sda[-1][1]

	def _verify_idle(self):
		if self._scl_state() == 1 and self._sda_state() == 1:
			return True

		return False

	def sda_trans(self, value, delay):
		self.sda.append((self.cycle_count+delay, value))
		self.cycle_count += delay

	def scl_trans(self, value, delay):
		self.scl.append((self.cycle_count+delay, value))
		self.cycle_count += delay

	def _print_bus(self):
		return "SDA:%d , SCL: %d" % (self.sda[-1][1], self.scl[-1][1])

	def send_start(self):
		if self._sda_state() == 0:
			self.sda_trans(1, 5)

		if self._scl_state() == 0:
			self.scl_trans(1, 5)

		if not self._verify_idle():
			raise ValueError("Tried to send a start with the bus not idle: %s" % self._print_bus())

		self.sda_trans(0, 20)
		self.scl_trans(0, 20)

	def send_bit(self, val):
		if self._scl_state() != 0:
			raise ValueError("Starting a bit when SCL is high.")

		self.sda_trans(val, self.half_baud)
		self.scl_trans(1, self.half_baud*2)
		self.scl_trans(0, self.half_baud)

	def delay(self, val):
		self.cycle_count += val

	def send_byte(self, val):
		val = val << 1
		
		for i in xrange(0,9):
			bitnum = 8-i
			bit = (val & (1 << bitnum)) >> bitnum
			self.send_bit(bit)

	def report(self):
		sda = map(lambda x: ('SDA', x[0], x[1]), self.sda)
		scl = map(lambda x: ('SCL', x[0], x[1]), self.scl)

		comb= sda + scl

		rep = sorted(comb, key=lambda x: x[1])

		for name,delay,val in rep:
			print "%d: %s=%d" % (delay, name, val)

	def write(self, addr, bytes):
		if not self._verify_idle():
			raise ValueError("Do not support repeated start yet")

		self.send_start()
		abyte = addr<<1

		self.send_byte(abyte)

		self.delay(1000)

		for byte in bytes:
			self.send_byte(byte)
			self.delay(1000)

	def read(self, addr):
		"""
		Send a (repeated) start along with an address with read indication
		"""
		self.send_start()
		abyte = addr<<1 | 1

		self.send_byte(abyte)

	def _gpsim_header(self):
		h = "stimulus asynchronous_stimulus\n"
		h += "initial_state 1\n"
		h += "start_cycle %d\n" % self.start
		h += "{ "

		return h

	def _gpsim_trailer(self, name):
		t = "name %s\n" % name
		t += "end\n"

		return t

	def to_gpsim(self, scl_name, sda_name):
		"""
		return two name stimuli definitions compatible with gpsim
		"""

		scl = self._gpsim_header()
		sda = self._gpsim_header()

		for delay, val in self.scl:
			scl += "%d, %d,\n" % (delay, val)

		scl = scl[:-2]
		scl += " }\n"

		for delay, val in self.sda:
			sda += "%d, %d,\n" % (delay, val)

		sda = sda[:-2]
		sda += " }\n"

		scl += self._gpsim_trailer(scl_name)
		sda += self._gpsim_trailer(sda_name)

		return {'SDA':sda, 'SCL':scl}

	@classmethod
	def checksum(cls, bytes):
		s = 0

		for byte in bytes:
			s += byte

		return (~s) + 1

seq = I2CMasterSequence(1000)

cmd = [10, 0, 0]
cmd.append(seq.checksum(cmd))

seq.write(10, cmd)
seq.read(10)

vals = seq.to_gpsim('scl_seq', 'sda_seq')

print vals['SCL']
print vals['SDA']


#physical_part.py
#An object representing a unique physical circuit part that can be sold by
#zero or more sellers

def convert(arg, **kwargs):
	if isinstance(arg, PhysicalPart):
		return arg

	if not isinstance(arg, dict):
		raise ValueError("conversion to physical_part from anything other than dict not supported")

	req = ['manu', 'mpn']
	for r in req:
		if r not in arg:
			raise ValueError('invalid dictionary used to initialize physical_part')

	desc = None
	if 'desc' in arg:
		desc = arg['desc']

	offers = []
	if 'offers' in arg:
		offers = arg['offers']

	return PhysicalPart(arg['manu'], arg['mpn'], offers, desc=desc)

def default_formatter(arg, **kwargs):
	return str(arg)

class PhysicalPart:
	"""
	A unique physical circuit component that can be purchased from zero or more suppliers.
	This would correspond to a single BOM line on a Bill of Materials for a circuit board.
	"""

	def __init__(self, manu, mpn, offers, desc=None):		
		self.offers = offers
		self.manu = manu
		self.mpn = mpn
		self.desc = desc

	def best_price(self, quantity, requirements):
		valid_offers = filter(lambda x: requirements.validate(x, quantity), self.offers)
		prices = map(lambda x: (x.best_price(quantity), x), valid_offers)		
		valid_prices = filter(lambda x: x[0] is not None, prices)

		sorted_prices = sorted(valid_prices, key=lambda x:x[0])

		if len(sorted_prices) > 0:
			return sorted_prices[0]

		return None

	def __str__(self):
		manu = self.manu
		mpn = self.mpn
		desc = self.desc

		#Make sure we get rid of non-ascii characters in case this 
		#string is printed since it would choke.
		if isinstance(manu, unicode):
			manu = manu.encode('ascii','ignore')
		if isinstance(mpn, unicode):
			mpn = mpn.encode('ascii','ignore')
		if isinstance(desc, unicode):
			desc = desc.encode('ascii','ignore')

		header = "Manufacturer: %s\nMPN: %s" % (manu, mpn)

		if self.desc is not None:
			header += "\nDescription: %s" % desc

		offers = map(str, self.offers)

		if len(offers) > 0:
			offerstr = "\n".join(offers)
			offerstr = "  " + offerstr.replace('\n', '\n  ')
			header += '\nOffers\n'
			header += offerstr

		return header
		


#template.py
#Utilities for producing skeleton code for application modules, unit tests,
#etc. using the Cheetah templating library.

import os.path
import os
from paths import MomoPaths
import tempfile
import Cheetah.Template

class RecursiveTemplate:
	"""
	Either a single Cheetah template document or a 
	directory containing one or more Cheetah template documents.
	In either case, the templates are filled in using supplied
	variables and if name points to a directory under /config/templates
	that directory structure is preserved in the output.
	"""

	def __init__(self, name, rename=None):
		self.objs = []
		self.name = name
		self.recursive = False
		self.rename = rename

		self._build_path()
		self._check_name()
	
	def _build_path(self):
		self.basepath = MomoPaths().templates

	def _check_name(self):
		path = os.path.join(self.basepath, self.name)
		if not os.path.exists(path):
			raise ValueError('%s does not exist' % path)

		if os.path.isdir(path):
			self.recursive = True

	def add(self, obj):
		self.objs.append(obj)

	def clear(self):
		self.objs = []

	def format_string(self, string):
		templ = Cheetah.Template.Template(source=string, searchList=self.objs)

		return str(templ)

	def format_file(self, file_in, file_out):
		templ = Cheetah.Template.Template(file = file_in, searchList=self.objs)

		with open(file_out, "w") as f:
			f.write(str(templ))

	def _ensure_path(self,path):
		if not os.path.exists(path):
			os.makedirs(path)

	def format_temp(self):
		"""
		For a nonrecursive template, render its output to a temporary file
		and return the file name.  The caller is responible for deleting the file
		when it is no longer needed.
		"""

		if self.recursive:
			raise ValueError("Cannot call format_temp on recursive templates")

		fh, outpath = tempfile.mkstemp();
		os.close(fh)

		self.format_file(os.path.join(self.basepath, self.name), outpath)
		return outpath

	def format(self, file_in, output_dir):
		"""
		Given a relative path from the templates directory (file_in), construct
		a file with the same relative path in output_dir by filling in the template
		and filling in file name (if it contains placeholders)
		"""

		inpath = os.path.join(self.basepath, file_in)

		if self.rename is None:
			path = os.path.join(output_dir, file_in)
		else:
			relpath = os.path.relpath(inpath, os.path.join(self.basepath, self.name))
			path = os.path.join(output_dir, self.rename, relpath)

		filled_path = self.format_string(path)
		dname = os.path.dirname(path)

		self._ensure_path(dname)
		self.format_file(inpath, filled_path)

	def render(self, output_dir):
		if not self.recursive:
			self.format(self.name, output_dir)
		else:
			indir = os.path.join(self.basepath, self.name)
			for dirpath, dirs, files in os.walk(indir):
				for f in files:
					inpath = os.path.relpath(os.path.join(dirpath, f), start=self.basepath)
					
					self.format(file_in=inpath, output_dir=output_dir)


from pymomo.utilities import typedargs
from pymomo.utilities.typedargs import type_system
import unittest
from nose.tools import *
import os.path
from pymomo.exceptions import *

def test_splitting():
	base, is_complex, subs = type_system.split_type('map(string, integer)')
	assert base == 'map'
	assert is_complex == True
	assert len(subs) == 2
	assert subs[0] == 'string'
	assert subs[1] == 'integer'

def test_map_type():
	mapper = type_system.get_type('map(string, string)')

def test_map_formatting():
	val = {'hello': 5}

	formatted = type_system.format_value(val, 'map(string, integer)')
	assert formatted == 'hello: 5'

def test_list_type():
	mapper = type_system.get_type('list(integer)')

def test_list_formatting():
	val = [10, 15]

	formatted = type_system.format_value(val, 'list(integer)')
	assert formatted == "10\n15"
